/*
 * Copyright (C) 2026 Alexander Borisov
 *
 * Author: Alexander Borisov <borisov@lexbor.com>
 */

/*
 * Caution!
 * This file generated by the script "utils/lexbor/css/names.py"!
 * Do not change this file!
 */


#ifndef LXB_CSS_AT_RULE_TYPES_H
#define LXB_CSS_AT_RULE_TYPES_H


static const lxb_css_syntax_cb_at_rule_t lxb_css_at_rule__undef_cb = {
    .prelude = lxb_css_at_rule__undef_prelude,
    .prelude_end = lxb_css_at_rule__undef_prelude_end,
    .block = lxb_css_at_rule__undef_block,
    .cb.failed = lxb_css_at_rule__undef_prelude_failed,
    .cb.end = lxb_css_at_rule__undef_end
};

static const lxb_css_syntax_cb_at_rule_t lxb_css_at_rule__custom_cb = {
    .prelude = lxb_css_at_rule__custom_prelude,
    .prelude_end = lxb_css_at_rule__custom_prelude_end,
    .block = lxb_css_at_rule__custom_block,
    .cb.failed = lxb_css_at_rule__custom_prelude_failed,
    .cb.end = lxb_css_at_rule__custom_end
};

static const lxb_css_syntax_cb_at_rule_t lxb_css_at_rule_font_face_cb = {
    .prelude = lxb_css_at_rule_font_face_prelude,
    .prelude_end = lxb_css_at_rule_font_face_prelude_end,
    .block = lxb_css_at_rule_font_face_block,
    .cb.failed = lxb_css_at_rule_font_face_prelude_failed,
    .cb.end = lxb_css_at_rule_font_face_end
};

static const lxb_css_syntax_cb_at_rule_t lxb_css_at_rule_media_cb = {
    .prelude = lxb_css_at_rule_media_prelude,
    .prelude_end = lxb_css_at_rule_media_prelude_end,
    .block = lxb_css_at_rule_media_block,
    .cb.failed = lxb_css_at_rule_media_prelude_failed,
    .cb.end = lxb_css_at_rule_media_end
};

static const lxb_css_syntax_cb_at_rule_t lxb_css_at_rule_namespace_cb = {
    .prelude = lxb_css_at_rule_namespace_prelude,
    .prelude_end = lxb_css_at_rule_namespace_prelude_end,
    .block = lxb_css_at_rule_namespace_block,
    .cb.failed = lxb_css_at_rule_namespace_prelude_failed,
    .cb.end = lxb_css_at_rule_namespace_end
};



#endif /* LXB_CSS_AT_RULE_TYPES_H */
