/*
   +----------------------------------------------------------------------+
   | Copyright (c) The PHP Group                                          |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
   | https://www.php.net/license/3_01.txt                                 |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
*/

#ifndef MBFL_SINGLEBYTE_H
#define MBFL_SINGLEBYTE_H

#include "mbfilter.h"

extern const mbfl_encoding mbfl_encoding_ascii;
extern const mbfl_encoding mbfl_encoding_8859_1;
extern const mbfl_encoding mbfl_encoding_8859_2;
extern const mbfl_encoding mbfl_encoding_8859_3;
extern const mbfl_encoding mbfl_encoding_8859_4;
extern const mbfl_encoding mbfl_encoding_8859_5;
extern const mbfl_encoding mbfl_encoding_8859_6;
extern const mbfl_encoding mbfl_encoding_8859_7;
extern const mbfl_encoding mbfl_encoding_8859_8;
extern const mbfl_encoding mbfl_encoding_8859_9;
extern const mbfl_encoding mbfl_encoding_8859_10;
extern const mbfl_encoding mbfl_encoding_8859_13;
extern const mbfl_encoding mbfl_encoding_8859_14;
extern const mbfl_encoding mbfl_encoding_8859_15;
extern const mbfl_encoding mbfl_encoding_8859_16;
extern const mbfl_encoding mbfl_encoding_cp1251;
extern const mbfl_encoding mbfl_encoding_cp1252;
extern const mbfl_encoding mbfl_encoding_cp1254;
extern const mbfl_encoding mbfl_encoding_cp866;
extern const mbfl_encoding mbfl_encoding_cp850;
extern const mbfl_encoding mbfl_encoding_koi8r;
extern const mbfl_encoding mbfl_encoding_koi8u;
extern const mbfl_encoding mbfl_encoding_armscii8;

#endif /* MBFL_SINGLEBYTE_H */
