/*
 * Copyright (C) 2026 Alexander Borisov
 *
 * Author: Alexander Borisov <borisov@lexbor.com>
 */

/*
 * Caution!
 * This file generated by the script "utils/lexbor/encoding/multi-byte.pl"!
 * Do not change this file!
 */

#ifndef LEXBOR_ENCODING_MULTI_H
#define LEXBOR_ENCODING_MULTI_H

#ifdef __cplusplus
extern "C" {
#endif

#include "lexbor/encoding/base.h"

#ifndef LEXBOR_DISABLE_INTERNAL_EXTERN
LXB_EXTERN lxb_codepoint_t lxb_encoding_multi_big5_map[19782];
LXB_EXTERN lxb_codepoint_t lxb_encoding_multi_euc_kr_map[23750];
LXB_EXTERN lxb_codepoint_t lxb_encoding_multi_gb18030_map[23940];
LXB_EXTERN lxb_codepoint_t lxb_encoding_multi_iso_2022_jp_katakana_map[63];
LXB_EXTERN lxb_codepoint_t lxb_encoding_multi_jis0212_map[7211];
LXB_EXTERN lxb_codepoint_t lxb_encoding_multi_jis0208_map[11104];

LXB_EXTERN uint16_t lxb_encoding_multi_big5_167_1106_map[939];
LXB_EXTERN uint16_t lxb_encoding_multi_big5_8211_40882_map[32671];
LXB_EXTERN uint16_t lxb_encoding_multi_big5_64012_65518_map[1506];
LXB_EXTERN uint16_t lxb_encoding_multi_big5_131210_172369_map[41159];
LXB_EXTERN uint16_t lxb_encoding_multi_big5_194708_194727_map[19];

LXB_EXTERN uint16_t lxb_encoding_multi_euc_kr_161_1106_map[945];
LXB_EXTERN uint16_t lxb_encoding_multi_euc_kr_8213_13278_map[5065];
LXB_EXTERN uint16_t lxb_encoding_multi_euc_kr_19968_55204_map[35236];
LXB_EXTERN uint16_t lxb_encoding_multi_euc_kr_63744_65511_map[1767];

LXB_EXTERN uint16_t lxb_encoding_multi_gb18030_164_1106_map[942];
LXB_EXTERN uint16_t lxb_encoding_multi_gb18030_7743_40892_map[33149];
LXB_EXTERN uint16_t lxb_encoding_multi_gb18030_57344_65510_map[8166];

LXB_EXTERN uint16_t lxb_encoding_multi_iso_2022_jp_katakana_12289_12541_map[252];

LXB_EXTERN uint16_t lxb_encoding_multi_jis0212_161_1120_map[959];
LXB_EXTERN uint16_t lxb_encoding_multi_jis0212_8470_8483_map[13];
LXB_EXTERN uint16_t lxb_encoding_multi_jis0212_19970_40870_map[20900];
LXB_EXTERN uint16_t lxb_encoding_multi_jis0212_65374_65375_map[1];

LXB_EXTERN uint16_t lxb_encoding_multi_jis0208_167_1106_map[939];
LXB_EXTERN uint16_t lxb_encoding_multi_jis0208_8208_13262_map[5054];
LXB_EXTERN uint16_t lxb_encoding_multi_jis0208_19968_40865_map[20897];
LXB_EXTERN uint16_t lxb_encoding_multi_jis0208_63785_65510_map[1725];
#endif /* !LEXBOR_DISABLE_INTERNAL_EXTERN */


#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* LEXBOR_ENCODING_MULTI_H */
