<?php
return function (mixed $in = null, array $options = []) {
 $a = null;
 $b = '';
 $c = '';
 $d = fn() => '';
 $e = fn() => '';
 $f = fn() => '';
 $g = fn() => '';
 return ''
  .LR::p(null, '', null, [], '')
  .LR::hbbch([null], $d)
  .LR::hbbch([null], $e)
  .LR::hbbch([null], $f)
  .LR::hbbch([null], $g);
};
