/*
 * Copyright (C) 2026 Alexander Borisov
 *
 * Author: Alexander Borisov <borisov@lexbor.com>
 */

/*
 * Caution!
 * This file generated by the script "utils/lexbor/css/names.py"!
 * Do not change this file!
 */


#ifndef LXB_CSS_VALUE_RES_H
#define LXB_CSS_VALUE_RES_H

#include "lexbor/core/shs.h"
#include "lexbor/css/value/const.h"

static const lxb_css_data_t lxb_css_value_data[LXB_CSS_VALUE__LAST_ENTRY] = 
{
    {(lxb_char_t *) "_undef", 6, LXB_CSS_VALUE__UNDEF},
    {(lxb_char_t *) "initial", 7, LXB_CSS_VALUE_INITIAL},
    {(lxb_char_t *) "inherit", 7, LXB_CSS_VALUE_INHERIT},
    {(lxb_char_t *) "unset", 5, LXB_CSS_VALUE_UNSET},
    {(lxb_char_t *) "revert", 6, LXB_CSS_VALUE_REVERT},
    {(lxb_char_t *) "flex-start", 10, LXB_CSS_VALUE_FLEX_START},
    {(lxb_char_t *) "flex-end", 8, LXB_CSS_VALUE_FLEX_END},
    {(lxb_char_t *) "center", 6, LXB_CSS_VALUE_CENTER},
    {(lxb_char_t *) "space-between", 13, LXB_CSS_VALUE_SPACE_BETWEEN},
    {(lxb_char_t *) "space-around", 12, LXB_CSS_VALUE_SPACE_AROUND},
    {(lxb_char_t *) "stretch", 7, LXB_CSS_VALUE_STRETCH},
    {(lxb_char_t *) "baseline", 8, LXB_CSS_VALUE_BASELINE},
    {(lxb_char_t *) "auto", 4, LXB_CSS_VALUE_AUTO},
    {(lxb_char_t *) "text-bottom", 11, LXB_CSS_VALUE_TEXT_BOTTOM},
    {(lxb_char_t *) "alphabetic", 10, LXB_CSS_VALUE_ALPHABETIC},
    {(lxb_char_t *) "ideographic", 11, LXB_CSS_VALUE_IDEOGRAPHIC},
    {(lxb_char_t *) "middle", 6, LXB_CSS_VALUE_MIDDLE},
    {(lxb_char_t *) "central", 7, LXB_CSS_VALUE_CENTRAL},
    {(lxb_char_t *) "mathematical", 12, LXB_CSS_VALUE_MATHEMATICAL},
    {(lxb_char_t *) "text-top", 8, LXB_CSS_VALUE_TEXT_TOP},
    {(lxb_char_t *) "_length", 7, LXB_CSS_VALUE__LENGTH},
    {(lxb_char_t *) "_percentage", 11, LXB_CSS_VALUE__PERCENTAGE},
    {(lxb_char_t *) "sub", 3, LXB_CSS_VALUE_SUB},
    {(lxb_char_t *) "super", 5, LXB_CSS_VALUE_SUPER},
    {(lxb_char_t *) "top", 3, LXB_CSS_VALUE_TOP},
    {(lxb_char_t *) "bottom", 6, LXB_CSS_VALUE_BOTTOM},
    {(lxb_char_t *) "first", 5, LXB_CSS_VALUE_FIRST},
    {(lxb_char_t *) "last", 4, LXB_CSS_VALUE_LAST},
    {(lxb_char_t *) "thin", 4, LXB_CSS_VALUE_THIN},
    {(lxb_char_t *) "medium", 6, LXB_CSS_VALUE_MEDIUM},
    {(lxb_char_t *) "thick", 5, LXB_CSS_VALUE_THICK},
    {(lxb_char_t *) "none", 4, LXB_CSS_VALUE_NONE},
    {(lxb_char_t *) "hidden", 6, LXB_CSS_VALUE_HIDDEN},
    {(lxb_char_t *) "dotted", 6, LXB_CSS_VALUE_DOTTED},
    {(lxb_char_t *) "dashed", 6, LXB_CSS_VALUE_DASHED},
    {(lxb_char_t *) "solid", 5, LXB_CSS_VALUE_SOLID},
    {(lxb_char_t *) "double", 6, LXB_CSS_VALUE_DOUBLE},
    {(lxb_char_t *) "groove", 6, LXB_CSS_VALUE_GROOVE},
    {(lxb_char_t *) "ridge", 5, LXB_CSS_VALUE_RIDGE},
    {(lxb_char_t *) "inset", 5, LXB_CSS_VALUE_INSET},
    {(lxb_char_t *) "outset", 6, LXB_CSS_VALUE_OUTSET},
    {(lxb_char_t *) "content-box", 11, LXB_CSS_VALUE_CONTENT_BOX},
    {(lxb_char_t *) "border-box", 10, LXB_CSS_VALUE_BORDER_BOX},
    {(lxb_char_t *) "inline-start", 12, LXB_CSS_VALUE_INLINE_START},
    {(lxb_char_t *) "inline-end", 10, LXB_CSS_VALUE_INLINE_END},
    {(lxb_char_t *) "block-start", 11, LXB_CSS_VALUE_BLOCK_START},
    {(lxb_char_t *) "block-end", 9, LXB_CSS_VALUE_BLOCK_END},
    {(lxb_char_t *) "left", 4, LXB_CSS_VALUE_LEFT},
    {(lxb_char_t *) "right", 5, LXB_CSS_VALUE_RIGHT},
    {(lxb_char_t *) "currentcolor", 12, LXB_CSS_VALUE_CURRENTCOLOR},
    {(lxb_char_t *) "transparent", 11, LXB_CSS_VALUE_TRANSPARENT},
    {(lxb_char_t *) "hex", 3, LXB_CSS_VALUE_HEX},
    {(lxb_char_t *) "aliceblue", 9, LXB_CSS_VALUE_ALICEBLUE},
    {(lxb_char_t *) "antiquewhite", 12, LXB_CSS_VALUE_ANTIQUEWHITE},
    {(lxb_char_t *) "aqua", 4, LXB_CSS_VALUE_AQUA},
    {(lxb_char_t *) "aquamarine", 10, LXB_CSS_VALUE_AQUAMARINE},
    {(lxb_char_t *) "azure", 5, LXB_CSS_VALUE_AZURE},
    {(lxb_char_t *) "beige", 5, LXB_CSS_VALUE_BEIGE},
    {(lxb_char_t *) "bisque", 6, LXB_CSS_VALUE_BISQUE},
    {(lxb_char_t *) "black", 5, LXB_CSS_VALUE_BLACK},
    {(lxb_char_t *) "blanchedalmond", 14, LXB_CSS_VALUE_BLANCHEDALMOND},
    {(lxb_char_t *) "blue", 4, LXB_CSS_VALUE_BLUE},
    {(lxb_char_t *) "blueviolet", 10, LXB_CSS_VALUE_BLUEVIOLET},
    {(lxb_char_t *) "brown", 5, LXB_CSS_VALUE_BROWN},
    {(lxb_char_t *) "burlywood", 9, LXB_CSS_VALUE_BURLYWOOD},
    {(lxb_char_t *) "cadetblue", 9, LXB_CSS_VALUE_CADETBLUE},
    {(lxb_char_t *) "chartreuse", 10, LXB_CSS_VALUE_CHARTREUSE},
    {(lxb_char_t *) "chocolate", 9, LXB_CSS_VALUE_CHOCOLATE},
    {(lxb_char_t *) "coral", 5, LXB_CSS_VALUE_CORAL},
    {(lxb_char_t *) "cornflowerblue", 14, LXB_CSS_VALUE_CORNFLOWERBLUE},
    {(lxb_char_t *) "cornsilk", 8, LXB_CSS_VALUE_CORNSILK},
    {(lxb_char_t *) "crimson", 7, LXB_CSS_VALUE_CRIMSON},
    {(lxb_char_t *) "cyan", 4, LXB_CSS_VALUE_CYAN},
    {(lxb_char_t *) "darkblue", 8, LXB_CSS_VALUE_DARKBLUE},
    {(lxb_char_t *) "darkcyan", 8, LXB_CSS_VALUE_DARKCYAN},
    {(lxb_char_t *) "darkgoldenrod", 13, LXB_CSS_VALUE_DARKGOLDENROD},
    {(lxb_char_t *) "darkgray", 8, LXB_CSS_VALUE_DARKGRAY},
    {(lxb_char_t *) "darkgreen", 9, LXB_CSS_VALUE_DARKGREEN},
    {(lxb_char_t *) "darkgrey", 8, LXB_CSS_VALUE_DARKGREY},
    {(lxb_char_t *) "darkkhaki", 9, LXB_CSS_VALUE_DARKKHAKI},
    {(lxb_char_t *) "darkmagenta", 11, LXB_CSS_VALUE_DARKMAGENTA},
    {(lxb_char_t *) "darkolivegreen", 14, LXB_CSS_VALUE_DARKOLIVEGREEN},
    {(lxb_char_t *) "darkorange", 10, LXB_CSS_VALUE_DARKORANGE},
    {(lxb_char_t *) "darkorchid", 10, LXB_CSS_VALUE_DARKORCHID},
    {(lxb_char_t *) "darkred", 7, LXB_CSS_VALUE_DARKRED},
    {(lxb_char_t *) "darksalmon", 10, LXB_CSS_VALUE_DARKSALMON},
    {(lxb_char_t *) "darkseagreen", 12, LXB_CSS_VALUE_DARKSEAGREEN},
    {(lxb_char_t *) "darkslateblue", 13, LXB_CSS_VALUE_DARKSLATEBLUE},
    {(lxb_char_t *) "darkslategray", 13, LXB_CSS_VALUE_DARKSLATEGRAY},
    {(lxb_char_t *) "darkslategrey", 13, LXB_CSS_VALUE_DARKSLATEGREY},
    {(lxb_char_t *) "darkturquoise", 13, LXB_CSS_VALUE_DARKTURQUOISE},
    {(lxb_char_t *) "darkviolet", 10, LXB_CSS_VALUE_DARKVIOLET},
    {(lxb_char_t *) "deeppink", 8, LXB_CSS_VALUE_DEEPPINK},
    {(lxb_char_t *) "deepskyblue", 11, LXB_CSS_VALUE_DEEPSKYBLUE},
    {(lxb_char_t *) "dimgray", 7, LXB_CSS_VALUE_DIMGRAY},
    {(lxb_char_t *) "dimgrey", 7, LXB_CSS_VALUE_DIMGREY},
    {(lxb_char_t *) "dodgerblue", 10, LXB_CSS_VALUE_DODGERBLUE},
    {(lxb_char_t *) "firebrick", 9, LXB_CSS_VALUE_FIREBRICK},
    {(lxb_char_t *) "floralwhite", 11, LXB_CSS_VALUE_FLORALWHITE},
    {(lxb_char_t *) "forestgreen", 11, LXB_CSS_VALUE_FORESTGREEN},
    {(lxb_char_t *) "fuchsia", 7, LXB_CSS_VALUE_FUCHSIA},
    {(lxb_char_t *) "gainsboro", 9, LXB_CSS_VALUE_GAINSBORO},
    {(lxb_char_t *) "ghostwhite", 10, LXB_CSS_VALUE_GHOSTWHITE},
    {(lxb_char_t *) "gold", 4, LXB_CSS_VALUE_GOLD},
    {(lxb_char_t *) "goldenrod", 9, LXB_CSS_VALUE_GOLDENROD},
    {(lxb_char_t *) "gray", 4, LXB_CSS_VALUE_GRAY},
    {(lxb_char_t *) "green", 5, LXB_CSS_VALUE_GREEN},
    {(lxb_char_t *) "greenyellow", 11, LXB_CSS_VALUE_GREENYELLOW},
    {(lxb_char_t *) "grey", 4, LXB_CSS_VALUE_GREY},
    {(lxb_char_t *) "honeydew", 8, LXB_CSS_VALUE_HONEYDEW},
    {(lxb_char_t *) "hotpink", 7, LXB_CSS_VALUE_HOTPINK},
    {(lxb_char_t *) "indianred", 9, LXB_CSS_VALUE_INDIANRED},
    {(lxb_char_t *) "indigo", 6, LXB_CSS_VALUE_INDIGO},
    {(lxb_char_t *) "ivory", 5, LXB_CSS_VALUE_IVORY},
    {(lxb_char_t *) "khaki", 5, LXB_CSS_VALUE_KHAKI},
    {(lxb_char_t *) "lavender", 8, LXB_CSS_VALUE_LAVENDER},
    {(lxb_char_t *) "lavenderblush", 13, LXB_CSS_VALUE_LAVENDERBLUSH},
    {(lxb_char_t *) "lawngreen", 9, LXB_CSS_VALUE_LAWNGREEN},
    {(lxb_char_t *) "lemonchiffon", 12, LXB_CSS_VALUE_LEMONCHIFFON},
    {(lxb_char_t *) "lightblue", 9, LXB_CSS_VALUE_LIGHTBLUE},
    {(lxb_char_t *) "lightcoral", 10, LXB_CSS_VALUE_LIGHTCORAL},
    {(lxb_char_t *) "lightcyan", 9, LXB_CSS_VALUE_LIGHTCYAN},
    {(lxb_char_t *) "lightgoldenrodyellow", 20, LXB_CSS_VALUE_LIGHTGOLDENRODYELLOW},
    {(lxb_char_t *) "lightgray", 9, LXB_CSS_VALUE_LIGHTGRAY},
    {(lxb_char_t *) "lightgreen", 10, LXB_CSS_VALUE_LIGHTGREEN},
    {(lxb_char_t *) "lightgrey", 9, LXB_CSS_VALUE_LIGHTGREY},
    {(lxb_char_t *) "lightpink", 9, LXB_CSS_VALUE_LIGHTPINK},
    {(lxb_char_t *) "lightsalmon", 11, LXB_CSS_VALUE_LIGHTSALMON},
    {(lxb_char_t *) "lightseagreen", 13, LXB_CSS_VALUE_LIGHTSEAGREEN},
    {(lxb_char_t *) "lightskyblue", 12, LXB_CSS_VALUE_LIGHTSKYBLUE},
    {(lxb_char_t *) "lightslategray", 14, LXB_CSS_VALUE_LIGHTSLATEGRAY},
    {(lxb_char_t *) "lightslategrey", 14, LXB_CSS_VALUE_LIGHTSLATEGREY},
    {(lxb_char_t *) "lightsteelblue", 14, LXB_CSS_VALUE_LIGHTSTEELBLUE},
    {(lxb_char_t *) "lightyellow", 11, LXB_CSS_VALUE_LIGHTYELLOW},
    {(lxb_char_t *) "lime", 4, LXB_CSS_VALUE_LIME},
    {(lxb_char_t *) "limegreen", 9, LXB_CSS_VALUE_LIMEGREEN},
    {(lxb_char_t *) "linen", 5, LXB_CSS_VALUE_LINEN},
    {(lxb_char_t *) "magenta", 7, LXB_CSS_VALUE_MAGENTA},
    {(lxb_char_t *) "maroon", 6, LXB_CSS_VALUE_MAROON},
    {(lxb_char_t *) "mediumaquamarine", 16, LXB_CSS_VALUE_MEDIUMAQUAMARINE},
    {(lxb_char_t *) "mediumblue", 10, LXB_CSS_VALUE_MEDIUMBLUE},
    {(lxb_char_t *) "mediumorchid", 12, LXB_CSS_VALUE_MEDIUMORCHID},
    {(lxb_char_t *) "mediumpurple", 12, LXB_CSS_VALUE_MEDIUMPURPLE},
    {(lxb_char_t *) "mediumseagreen", 14, LXB_CSS_VALUE_MEDIUMSEAGREEN},
    {(lxb_char_t *) "mediumslateblue", 15, LXB_CSS_VALUE_MEDIUMSLATEBLUE},
    {(lxb_char_t *) "mediumspringgreen", 17, LXB_CSS_VALUE_MEDIUMSPRINGGREEN},
    {(lxb_char_t *) "mediumturquoise", 15, LXB_CSS_VALUE_MEDIUMTURQUOISE},
    {(lxb_char_t *) "mediumvioletred", 15, LXB_CSS_VALUE_MEDIUMVIOLETRED},
    {(lxb_char_t *) "midnightblue", 12, LXB_CSS_VALUE_MIDNIGHTBLUE},
    {(lxb_char_t *) "mintcream", 9, LXB_CSS_VALUE_MINTCREAM},
    {(lxb_char_t *) "mistyrose", 9, LXB_CSS_VALUE_MISTYROSE},
    {(lxb_char_t *) "moccasin", 8, LXB_CSS_VALUE_MOCCASIN},
    {(lxb_char_t *) "navajowhite", 11, LXB_CSS_VALUE_NAVAJOWHITE},
    {(lxb_char_t *) "navy", 4, LXB_CSS_VALUE_NAVY},
    {(lxb_char_t *) "oldlace", 7, LXB_CSS_VALUE_OLDLACE},
    {(lxb_char_t *) "olive", 5, LXB_CSS_VALUE_OLIVE},
    {(lxb_char_t *) "olivedrab", 9, LXB_CSS_VALUE_OLIVEDRAB},
    {(lxb_char_t *) "orange", 6, LXB_CSS_VALUE_ORANGE},
    {(lxb_char_t *) "orangered", 9, LXB_CSS_VALUE_ORANGERED},
    {(lxb_char_t *) "orchid", 6, LXB_CSS_VALUE_ORCHID},
    {(lxb_char_t *) "palegoldenrod", 13, LXB_CSS_VALUE_PALEGOLDENROD},
    {(lxb_char_t *) "palegreen", 9, LXB_CSS_VALUE_PALEGREEN},
    {(lxb_char_t *) "paleturquoise", 13, LXB_CSS_VALUE_PALETURQUOISE},
    {(lxb_char_t *) "palevioletred", 13, LXB_CSS_VALUE_PALEVIOLETRED},
    {(lxb_char_t *) "papayawhip", 10, LXB_CSS_VALUE_PAPAYAWHIP},
    {(lxb_char_t *) "peachpuff", 9, LXB_CSS_VALUE_PEACHPUFF},
    {(lxb_char_t *) "peru", 4, LXB_CSS_VALUE_PERU},
    {(lxb_char_t *) "pink", 4, LXB_CSS_VALUE_PINK},
    {(lxb_char_t *) "plum", 4, LXB_CSS_VALUE_PLUM},
    {(lxb_char_t *) "powderblue", 10, LXB_CSS_VALUE_POWDERBLUE},
    {(lxb_char_t *) "purple", 6, LXB_CSS_VALUE_PURPLE},
    {(lxb_char_t *) "rebeccapurple", 13, LXB_CSS_VALUE_REBECCAPURPLE},
    {(lxb_char_t *) "red", 3, LXB_CSS_VALUE_RED},
    {(lxb_char_t *) "rosybrown", 9, LXB_CSS_VALUE_ROSYBROWN},
    {(lxb_char_t *) "royalblue", 9, LXB_CSS_VALUE_ROYALBLUE},
    {(lxb_char_t *) "saddlebrown", 11, LXB_CSS_VALUE_SADDLEBROWN},
    {(lxb_char_t *) "salmon", 6, LXB_CSS_VALUE_SALMON},
    {(lxb_char_t *) "sandybrown", 10, LXB_CSS_VALUE_SANDYBROWN},
    {(lxb_char_t *) "seagreen", 8, LXB_CSS_VALUE_SEAGREEN},
    {(lxb_char_t *) "seashell", 8, LXB_CSS_VALUE_SEASHELL},
    {(lxb_char_t *) "sienna", 6, LXB_CSS_VALUE_SIENNA},
    {(lxb_char_t *) "silver", 6, LXB_CSS_VALUE_SILVER},
    {(lxb_char_t *) "skyblue", 7, LXB_CSS_VALUE_SKYBLUE},
    {(lxb_char_t *) "slateblue", 9, LXB_CSS_VALUE_SLATEBLUE},
    {(lxb_char_t *) "slategray", 9, LXB_CSS_VALUE_SLATEGRAY},
    {(lxb_char_t *) "slategrey", 9, LXB_CSS_VALUE_SLATEGREY},
    {(lxb_char_t *) "snow", 4, LXB_CSS_VALUE_SNOW},
    {(lxb_char_t *) "springgreen", 11, LXB_CSS_VALUE_SPRINGGREEN},
    {(lxb_char_t *) "steelblue", 9, LXB_CSS_VALUE_STEELBLUE},
    {(lxb_char_t *) "tan", 3, LXB_CSS_VALUE_TAN},
    {(lxb_char_t *) "teal", 4, LXB_CSS_VALUE_TEAL},
    {(lxb_char_t *) "thistle", 7, LXB_CSS_VALUE_THISTLE},
    {(lxb_char_t *) "tomato", 6, LXB_CSS_VALUE_TOMATO},
    {(lxb_char_t *) "turquoise", 9, LXB_CSS_VALUE_TURQUOISE},
    {(lxb_char_t *) "violet", 6, LXB_CSS_VALUE_VIOLET},
    {(lxb_char_t *) "wheat", 5, LXB_CSS_VALUE_WHEAT},
    {(lxb_char_t *) "white", 5, LXB_CSS_VALUE_WHITE},
    {(lxb_char_t *) "whitesmoke", 10, LXB_CSS_VALUE_WHITESMOKE},
    {(lxb_char_t *) "yellow", 6, LXB_CSS_VALUE_YELLOW},
    {(lxb_char_t *) "yellowgreen", 11, LXB_CSS_VALUE_YELLOWGREEN},
    {(lxb_char_t *) "Canvas", 6, LXB_CSS_VALUE_CANVAS},
    {(lxb_char_t *) "CanvasText", 10, LXB_CSS_VALUE_CANVASTEXT},
    {(lxb_char_t *) "LinkText", 8, LXB_CSS_VALUE_LINKTEXT},
    {(lxb_char_t *) "VisitedText", 11, LXB_CSS_VALUE_VISITEDTEXT},
    {(lxb_char_t *) "ActiveText", 10, LXB_CSS_VALUE_ACTIVETEXT},
    {(lxb_char_t *) "ButtonFace", 10, LXB_CSS_VALUE_BUTTONFACE},
    {(lxb_char_t *) "ButtonText", 10, LXB_CSS_VALUE_BUTTONTEXT},
    {(lxb_char_t *) "ButtonBorder", 12, LXB_CSS_VALUE_BUTTONBORDER},
    {(lxb_char_t *) "Field", 5, LXB_CSS_VALUE_FIELD},
    {(lxb_char_t *) "FieldText", 9, LXB_CSS_VALUE_FIELDTEXT},
    {(lxb_char_t *) "Highlight", 9, LXB_CSS_VALUE_HIGHLIGHT},
    {(lxb_char_t *) "HighlightText", 13, LXB_CSS_VALUE_HIGHLIGHTTEXT},
    {(lxb_char_t *) "SelectedItem", 12, LXB_CSS_VALUE_SELECTEDITEM},
    {(lxb_char_t *) "SelectedItemText", 16, LXB_CSS_VALUE_SELECTEDITEMTEXT},
    {(lxb_char_t *) "Mark", 4, LXB_CSS_VALUE_MARK},
    {(lxb_char_t *) "MarkText", 8, LXB_CSS_VALUE_MARKTEXT},
    {(lxb_char_t *) "GrayText", 8, LXB_CSS_VALUE_GRAYTEXT},
    {(lxb_char_t *) "AccentColor", 11, LXB_CSS_VALUE_ACCENTCOLOR},
    {(lxb_char_t *) "AccentColorText", 15, LXB_CSS_VALUE_ACCENTCOLORTEXT},
    {(lxb_char_t *) "rgb", 3, LXB_CSS_VALUE_RGB},
    {(lxb_char_t *) "rgba", 4, LXB_CSS_VALUE_RGBA},
    {(lxb_char_t *) "hsl", 3, LXB_CSS_VALUE_HSL},
    {(lxb_char_t *) "hsla", 4, LXB_CSS_VALUE_HSLA},
    {(lxb_char_t *) "hwb", 3, LXB_CSS_VALUE_HWB},
    {(lxb_char_t *) "lab", 3, LXB_CSS_VALUE_LAB},
    {(lxb_char_t *) "lch", 3, LXB_CSS_VALUE_LCH},
    {(lxb_char_t *) "oklab", 5, LXB_CSS_VALUE_OKLAB},
    {(lxb_char_t *) "oklch", 5, LXB_CSS_VALUE_OKLCH},
    {(lxb_char_t *) "color", 5, LXB_CSS_VALUE_COLOR},
    {(lxb_char_t *) "ltr", 3, LXB_CSS_VALUE_LTR},
    {(lxb_char_t *) "rtl", 3, LXB_CSS_VALUE_RTL},
    {(lxb_char_t *) "block", 5, LXB_CSS_VALUE_BLOCK},
    {(lxb_char_t *) "inline", 6, LXB_CSS_VALUE_INLINE},
    {(lxb_char_t *) "run-in", 6, LXB_CSS_VALUE_RUN_IN},
    {(lxb_char_t *) "flow", 4, LXB_CSS_VALUE_FLOW},
    {(lxb_char_t *) "flow-root", 9, LXB_CSS_VALUE_FLOW_ROOT},
    {(lxb_char_t *) "table", 5, LXB_CSS_VALUE_TABLE},
    {(lxb_char_t *) "flex", 4, LXB_CSS_VALUE_FLEX},
    {(lxb_char_t *) "grid", 4, LXB_CSS_VALUE_GRID},
    {(lxb_char_t *) "ruby", 4, LXB_CSS_VALUE_RUBY},
    {(lxb_char_t *) "list-item", 9, LXB_CSS_VALUE_LIST_ITEM},
    {(lxb_char_t *) "table-row-group", 15, LXB_CSS_VALUE_TABLE_ROW_GROUP},
    {(lxb_char_t *) "table-header-group", 18, LXB_CSS_VALUE_TABLE_HEADER_GROUP},
    {(lxb_char_t *) "table-footer-group", 18, LXB_CSS_VALUE_TABLE_FOOTER_GROUP},
    {(lxb_char_t *) "table-row", 9, LXB_CSS_VALUE_TABLE_ROW},
    {(lxb_char_t *) "table-cell", 10, LXB_CSS_VALUE_TABLE_CELL},
    {(lxb_char_t *) "table-column-group", 18, LXB_CSS_VALUE_TABLE_COLUMN_GROUP},
    {(lxb_char_t *) "table-column", 12, LXB_CSS_VALUE_TABLE_COLUMN},
    {(lxb_char_t *) "table-caption", 13, LXB_CSS_VALUE_TABLE_CAPTION},
    {(lxb_char_t *) "ruby-base", 9, LXB_CSS_VALUE_RUBY_BASE},
    {(lxb_char_t *) "ruby-text", 9, LXB_CSS_VALUE_RUBY_TEXT},
    {(lxb_char_t *) "ruby-base-container", 19, LXB_CSS_VALUE_RUBY_BASE_CONTAINER},
    {(lxb_char_t *) "ruby-text-container", 19, LXB_CSS_VALUE_RUBY_TEXT_CONTAINER},
    {(lxb_char_t *) "contents", 8, LXB_CSS_VALUE_CONTENTS},
    {(lxb_char_t *) "inline-block", 12, LXB_CSS_VALUE_INLINE_BLOCK},
    {(lxb_char_t *) "inline-table", 12, LXB_CSS_VALUE_INLINE_TABLE},
    {(lxb_char_t *) "inline-flex", 11, LXB_CSS_VALUE_INLINE_FLEX},
    {(lxb_char_t *) "inline-grid", 11, LXB_CSS_VALUE_INLINE_GRID},
    {(lxb_char_t *) "hanging", 7, LXB_CSS_VALUE_HANGING},
    {(lxb_char_t *) "content", 7, LXB_CSS_VALUE_CONTENT},
    {(lxb_char_t *) "row", 3, LXB_CSS_VALUE_ROW},
    {(lxb_char_t *) "row-reverse", 11, LXB_CSS_VALUE_ROW_REVERSE},
    {(lxb_char_t *) "column", 6, LXB_CSS_VALUE_COLUMN},
    {(lxb_char_t *) "column-reverse", 14, LXB_CSS_VALUE_COLUMN_REVERSE},
    {(lxb_char_t *) "_number", 7, LXB_CSS_VALUE__NUMBER},
    {(lxb_char_t *) "nowrap", 6, LXB_CSS_VALUE_NOWRAP},
    {(lxb_char_t *) "wrap", 4, LXB_CSS_VALUE_WRAP},
    {(lxb_char_t *) "wrap-reverse", 12, LXB_CSS_VALUE_WRAP_REVERSE},
    {(lxb_char_t *) "snap-block", 10, LXB_CSS_VALUE_SNAP_BLOCK},
    {(lxb_char_t *) "start", 5, LXB_CSS_VALUE_START},
    {(lxb_char_t *) "end", 3, LXB_CSS_VALUE_END},
    {(lxb_char_t *) "near", 4, LXB_CSS_VALUE_NEAR},
    {(lxb_char_t *) "snap-inline", 11, LXB_CSS_VALUE_SNAP_INLINE},
    {(lxb_char_t *) "_integer", 8, LXB_CSS_VALUE__INTEGER},
    {(lxb_char_t *) "region", 6, LXB_CSS_VALUE_REGION},
    {(lxb_char_t *) "page", 4, LXB_CSS_VALUE_PAGE},
    {(lxb_char_t *) "serif", 5, LXB_CSS_VALUE_SERIF},
    {(lxb_char_t *) "sans-serif", 10, LXB_CSS_VALUE_SANS_SERIF},
    {(lxb_char_t *) "cursive", 7, LXB_CSS_VALUE_CURSIVE},
    {(lxb_char_t *) "fantasy", 7, LXB_CSS_VALUE_FANTASY},
    {(lxb_char_t *) "monospace", 9, LXB_CSS_VALUE_MONOSPACE},
    {(lxb_char_t *) "system-ui", 9, LXB_CSS_VALUE_SYSTEM_UI},
    {(lxb_char_t *) "emoji", 5, LXB_CSS_VALUE_EMOJI},
    {(lxb_char_t *) "math", 4, LXB_CSS_VALUE_MATH},
    {(lxb_char_t *) "fangsong", 8, LXB_CSS_VALUE_FANGSONG},
    {(lxb_char_t *) "ui-serif", 8, LXB_CSS_VALUE_UI_SERIF},
    {(lxb_char_t *) "ui-sans-serif", 13, LXB_CSS_VALUE_UI_SANS_SERIF},
    {(lxb_char_t *) "ui-monospace", 12, LXB_CSS_VALUE_UI_MONOSPACE},
    {(lxb_char_t *) "ui-rounded", 10, LXB_CSS_VALUE_UI_ROUNDED},
    {(lxb_char_t *) "xx-small", 8, LXB_CSS_VALUE_XX_SMALL},
    {(lxb_char_t *) "x-small", 7, LXB_CSS_VALUE_X_SMALL},
    {(lxb_char_t *) "small", 5, LXB_CSS_VALUE_SMALL},
    {(lxb_char_t *) "large", 5, LXB_CSS_VALUE_LARGE},
    {(lxb_char_t *) "x-large", 7, LXB_CSS_VALUE_X_LARGE},
    {(lxb_char_t *) "xx-large", 8, LXB_CSS_VALUE_XX_LARGE},
    {(lxb_char_t *) "xxx-large", 9, LXB_CSS_VALUE_XXX_LARGE},
    {(lxb_char_t *) "larger", 6, LXB_CSS_VALUE_LARGER},
    {(lxb_char_t *) "smaller", 7, LXB_CSS_VALUE_SMALLER},
    {(lxb_char_t *) "normal", 6, LXB_CSS_VALUE_NORMAL},
    {(lxb_char_t *) "ultra-condensed", 15, LXB_CSS_VALUE_ULTRA_CONDENSED},
    {(lxb_char_t *) "extra-condensed", 15, LXB_CSS_VALUE_EXTRA_CONDENSED},
    {(lxb_char_t *) "condensed", 9, LXB_CSS_VALUE_CONDENSED},
    {(lxb_char_t *) "semi-condensed", 14, LXB_CSS_VALUE_SEMI_CONDENSED},
    {(lxb_char_t *) "semi-expanded", 13, LXB_CSS_VALUE_SEMI_EXPANDED},
    {(lxb_char_t *) "expanded", 8, LXB_CSS_VALUE_EXPANDED},
    {(lxb_char_t *) "extra-expanded", 14, LXB_CSS_VALUE_EXTRA_EXPANDED},
    {(lxb_char_t *) "ultra-expanded", 14, LXB_CSS_VALUE_ULTRA_EXPANDED},
    {(lxb_char_t *) "italic", 6, LXB_CSS_VALUE_ITALIC},
    {(lxb_char_t *) "oblique", 7, LXB_CSS_VALUE_OBLIQUE},
    {(lxb_char_t *) "bold", 4, LXB_CSS_VALUE_BOLD},
    {(lxb_char_t *) "bolder", 6, LXB_CSS_VALUE_BOLDER},
    {(lxb_char_t *) "lighter", 7, LXB_CSS_VALUE_LIGHTER},
    {(lxb_char_t *) "force-end", 9, LXB_CSS_VALUE_FORCE_END},
    {(lxb_char_t *) "allow-end", 9, LXB_CSS_VALUE_ALLOW_END},
    {(lxb_char_t *) "min-content", 11, LXB_CSS_VALUE_MIN_CONTENT},
    {(lxb_char_t *) "max-content", 11, LXB_CSS_VALUE_MAX_CONTENT},
    {(lxb_char_t *) "_angle", 6, LXB_CSS_VALUE__ANGLE},
    {(lxb_char_t *) "manual", 6, LXB_CSS_VALUE_MANUAL},
    {(lxb_char_t *) "loose", 5, LXB_CSS_VALUE_LOOSE},
    {(lxb_char_t *) "strict", 6, LXB_CSS_VALUE_STRICT},
    {(lxb_char_t *) "anywhere", 8, LXB_CSS_VALUE_ANYWHERE},
    {(lxb_char_t *) "visible", 7, LXB_CSS_VALUE_VISIBLE},
    {(lxb_char_t *) "clip", 4, LXB_CSS_VALUE_CLIP},
    {(lxb_char_t *) "scroll", 6, LXB_CSS_VALUE_SCROLL},
    {(lxb_char_t *) "break-word", 10, LXB_CSS_VALUE_BREAK_WORD},
    {(lxb_char_t *) "static", 6, LXB_CSS_VALUE_STATIC},
    {(lxb_char_t *) "relative", 8, LXB_CSS_VALUE_RELATIVE},
    {(lxb_char_t *) "absolute", 8, LXB_CSS_VALUE_ABSOLUTE},
    {(lxb_char_t *) "sticky", 6, LXB_CSS_VALUE_STICKY},
    {(lxb_char_t *) "fixed", 5, LXB_CSS_VALUE_FIXED},
    {(lxb_char_t *) "justify", 7, LXB_CSS_VALUE_JUSTIFY},
    {(lxb_char_t *) "match-parent", 12, LXB_CSS_VALUE_MATCH_PARENT},
    {(lxb_char_t *) "justify-all", 11, LXB_CSS_VALUE_JUSTIFY_ALL},
    {(lxb_char_t *) "all", 3, LXB_CSS_VALUE_ALL},
    {(lxb_char_t *) "digits", 6, LXB_CSS_VALUE_DIGITS},
    {(lxb_char_t *) "underline", 9, LXB_CSS_VALUE_UNDERLINE},
    {(lxb_char_t *) "overline", 8, LXB_CSS_VALUE_OVERLINE},
    {(lxb_char_t *) "line-through", 12, LXB_CSS_VALUE_LINE_THROUGH},
    {(lxb_char_t *) "blink", 5, LXB_CSS_VALUE_BLINK},
    {(lxb_char_t *) "wavy", 4, LXB_CSS_VALUE_WAVY},
    {(lxb_char_t *) "each-line", 9, LXB_CSS_VALUE_EACH_LINE},
    {(lxb_char_t *) "inter-word", 10, LXB_CSS_VALUE_INTER_WORD},
    {(lxb_char_t *) "inter-character", 15, LXB_CSS_VALUE_INTER_CHARACTER},
    {(lxb_char_t *) "mixed", 5, LXB_CSS_VALUE_MIXED},
    {(lxb_char_t *) "upright", 7, LXB_CSS_VALUE_UPRIGHT},
    {(lxb_char_t *) "sideways", 8, LXB_CSS_VALUE_SIDEWAYS},
    {(lxb_char_t *) "ellipsis", 8, LXB_CSS_VALUE_ELLIPSIS},
    {(lxb_char_t *) "capitalize", 10, LXB_CSS_VALUE_CAPITALIZE},
    {(lxb_char_t *) "uppercase", 9, LXB_CSS_VALUE_UPPERCASE},
    {(lxb_char_t *) "lowercase", 9, LXB_CSS_VALUE_LOWERCASE},
    {(lxb_char_t *) "full-width", 10, LXB_CSS_VALUE_FULL_WIDTH},
    {(lxb_char_t *) "full-size-kana", 14, LXB_CSS_VALUE_FULL_SIZE_KANA},
    {(lxb_char_t *) "embed", 5, LXB_CSS_VALUE_EMBED},
    {(lxb_char_t *) "isolate", 7, LXB_CSS_VALUE_ISOLATE},
    {(lxb_char_t *) "bidi-override", 13, LXB_CSS_VALUE_BIDI_OVERRIDE},
    {(lxb_char_t *) "isolate-override", 16, LXB_CSS_VALUE_ISOLATE_OVERRIDE},
    {(lxb_char_t *) "plaintext", 9, LXB_CSS_VALUE_PLAINTEXT},
    {(lxb_char_t *) "collapse", 8, LXB_CSS_VALUE_COLLAPSE},
    {(lxb_char_t *) "pre", 3, LXB_CSS_VALUE_PRE},
    {(lxb_char_t *) "pre-wrap", 8, LXB_CSS_VALUE_PRE_WRAP},
    {(lxb_char_t *) "break-spaces", 12, LXB_CSS_VALUE_BREAK_SPACES},
    {(lxb_char_t *) "pre-line", 8, LXB_CSS_VALUE_PRE_LINE},
    {(lxb_char_t *) "keep-all", 8, LXB_CSS_VALUE_KEEP_ALL},
    {(lxb_char_t *) "break-all", 9, LXB_CSS_VALUE_BREAK_ALL},
    {(lxb_char_t *) "both", 4, LXB_CSS_VALUE_BOTH},
    {(lxb_char_t *) "minimum", 7, LXB_CSS_VALUE_MINIMUM},
    {(lxb_char_t *) "maximum", 7, LXB_CSS_VALUE_MAXIMUM},
    {(lxb_char_t *) "clear", 5, LXB_CSS_VALUE_CLEAR},
    {(lxb_char_t *) "horizontal-tb", 13, LXB_CSS_VALUE_HORIZONTAL_TB},
    {(lxb_char_t *) "vertical-rl", 11, LXB_CSS_VALUE_VERTICAL_RL},
    {(lxb_char_t *) "vertical-lr", 11, LXB_CSS_VALUE_VERTICAL_LR},
    {(lxb_char_t *) "sideways-rl", 11, LXB_CSS_VALUE_SIDEWAYS_RL},
    {(lxb_char_t *) "sideways-lr", 11, LXB_CSS_VALUE_SIDEWAYS_LR}
};

static const lexbor_shs_entry_t lxb_css_value_shs[373] = 
{
    {NULL, NULL, 253, 0}, 
    {"salmon", (void *) LXB_CSS_VALUE_SALMON, 6, 2}, 
    {"canvas", (void *) LXB_CSS_VALUE_CANVAS, 6, 4}, 
    {"allow-end", (void *) LXB_CSS_VALUE_ALLOW_END, 9, 0}, 
    {"static", (void *) LXB_CSS_VALUE_STATIC, 6, 5}, 
    {"sticky", (void *) LXB_CSS_VALUE_STICKY, 6, 0}, 
    {"darkviolet", (void *) LXB_CSS_VALUE_DARKVIOLET, 10, 0}, 
    {"blue", (void *) LXB_CSS_VALUE_BLUE, 4, 1}, 
    {"end", (void *) LXB_CSS_VALUE_END, 3, 24}, 
    {"middle", (void *) LXB_CSS_VALUE_MIDDLE, 6, 32}, 
    {"rgb", (void *) LXB_CSS_VALUE_RGB, 3, 59}, 
    {"baseline", (void *) LXB_CSS_VALUE_BASELINE, 8, 69}, 
    {"mistyrose", (void *) LXB_CSS_VALUE_MISTYROSE, 9, 71}, 
    {"expanded", (void *) LXB_CSS_VALUE_EXPANDED, 8, 92}, 
    {"space-between", (void *) LXB_CSS_VALUE_SPACE_BETWEEN, 13, 0}, 
    {"mediumpurple", (void *) LXB_CSS_VALUE_MEDIUMPURPLE, 12, 99}, 
    {"bidi-override", (void *) LXB_CSS_VALUE_BIDI_OVERRIDE, 13, 0}, 
    {"bold", (void *) LXB_CSS_VALUE_BOLD, 4, 103}, 
    {"mixed", (void *) LXB_CSS_VALUE_MIXED, 5, 105}, 
    {"extra-expanded", (void *) LXB_CSS_VALUE_EXTRA_EXPANDED, 14, 119}, 
    {"auto", (void *) LXB_CSS_VALUE_AUTO, 4, 129}, 
    {"teal", (void *) LXB_CSS_VALUE_TEAL, 4, 130}, 
    {"lab", (void *) LXB_CSS_VALUE_LAB, 3, 131}, 
    {"break-word", (void *) LXB_CSS_VALUE_BREAK_WORD, 10, 0}, 
    {"beige", (void *) LXB_CSS_VALUE_BEIGE, 5, 0}, 
    {"mediumorchid", (void *) LXB_CSS_VALUE_MEDIUMORCHID, 12, 0}, 
    {"vertical-lr", (void *) LXB_CSS_VALUE_VERTICAL_LR, 11, 0}, 
    {"table-cell", (void *) LXB_CSS_VALUE_TABLE_CELL, 10, 135}, 
    {"pre-wrap", (void *) LXB_CSS_VALUE_PRE_WRAP, 8, 136}, 
    {"super", (void *) LXB_CSS_VALUE_SUPER, 5, 0}, 
    {"silver", (void *) LXB_CSS_VALUE_SILVER, 6, 137}, 
    {"hsl", (void *) LXB_CSS_VALUE_HSL, 3, 140}, 
    {"bisque", (void *) LXB_CSS_VALUE_BISQUE, 6, 53}, 
    {"darkblue", (void *) LXB_CSS_VALUE_DARKBLUE, 8, 0}, 
    {"hex", (void *) LXB_CSS_VALUE_HEX, 3, 0}, 
    {"darkorange", (void *) LXB_CSS_VALUE_DARKORANGE, 10, 143}, 
    {"lightcoral", (void *) LXB_CSS_VALUE_LIGHTCORAL, 10, 145}, 
    {"table-row", (void *) LXB_CSS_VALUE_TABLE_ROW, 9, 0}, 
    {"ridge", (void *) LXB_CSS_VALUE_RIDGE, 5, 149}, 
    {"violet", (void *) LXB_CSS_VALUE_VIOLET, 6, 156}, 
    {"italic", (void *) LXB_CSS_VALUE_ITALIC, 6, 157}, 
    {"relative", (void *) LXB_CSS_VALUE_RELATIVE, 8, 0}, 
    {"royalblue", (void *) LXB_CSS_VALUE_ROYALBLUE, 9, 179}, 
    {"highlighttext", (void *) LXB_CSS_VALUE_HIGHLIGHTTEXT, 13, 0}, 
    {"visitedtext", (void *) LXB_CSS_VALUE_VISITEDTEXT, 11, 193}, 
    {"ideographic", (void *) LXB_CSS_VALUE_IDEOGRAPHIC, 11, 0}, 
    {"outset", (void *) LXB_CSS_VALUE_OUTSET, 6, 194}, 
    {"azure", (void *) LXB_CSS_VALUE_AZURE, 5, 0}, 
    {"pink", (void *) LXB_CSS_VALUE_PINK, 4, 0}, 
    {"near", (void *) LXB_CSS_VALUE_NEAR, 4, 0}, 
    {"magenta", (void *) LXB_CSS_VALUE_MAGENTA, 7, 204}, 
    {"aliceblue", (void *) LXB_CSS_VALUE_ALICEBLUE, 9, 0}, 
    {"aquamarine", (void *) LXB_CSS_VALUE_AQUAMARINE, 10, 0}, 
    {"seagreen", (void *) LXB_CSS_VALUE_SEAGREEN, 8, 55}, 
    {"antiquewhite", (void *) LXB_CSS_VALUE_ANTIQUEWHITE, 12, 0}, 
    {"contents", (void *) LXB_CSS_VALUE_CONTENTS, 8, 56}, 
    {"lightpink", (void *) LXB_CSS_VALUE_LIGHTPINK, 9, 0}, 
    {"snap-block", (void *) LXB_CSS_VALUE_SNAP_BLOCK, 10, 0}, 
    {"white", (void *) LXB_CSS_VALUE_WHITE, 5, 0}, 
    {"embed", (void *) LXB_CSS_VALUE_EMBED, 5, 66}, 
    {"first", (void *) LXB_CSS_VALUE_FIRST, 5, 211}, 
    {"groove", (void *) LXB_CSS_VALUE_GROOVE, 6, 212}, 
    {"normal", (void *) LXB_CSS_VALUE_NORMAL, 6, 0}, 
    {"whitesmoke", (void *) LXB_CSS_VALUE_WHITESMOKE, 10, 0}, 
    {"fieldtext", (void *) LXB_CSS_VALUE_FIELDTEXT, 9, 221}, 
    {"flex-start", (void *) LXB_CSS_VALUE_FLEX_START, 10, 227}, 
    {"slategray", (void *) LXB_CSS_VALUE_SLATEGRAY, 9, 67}, 
    {"slategrey", (void *) LXB_CSS_VALUE_SLATEGREY, 9, 68}, 
    {"darkmagenta", (void *) LXB_CSS_VALUE_DARKMAGENTA, 11, 0}, 
    {"sandybrown", (void *) LXB_CSS_VALUE_SANDYBROWN, 10, 0}, 
    {"field", (void *) LXB_CSS_VALUE_FIELD, 5, 234}, 
    {"monospace", (void *) LXB_CSS_VALUE_MONOSPACE, 9, 78}, 
    {"color", (void *) LXB_CSS_VALUE_COLOR, 5, 236}, 
    {"center", (void *) LXB_CSS_VALUE_CENTER, 6, 240}, 
    {"force-end", (void *) LXB_CSS_VALUE_FORCE_END, 9, 0}, 
    {"solid", (void *) LXB_CSS_VALUE_SOLID, 5, 243}, 
    {"gold", (void *) LXB_CSS_VALUE_GOLD, 4, 248}, 
    {"peachpuff", (void *) LXB_CSS_VALUE_PEACHPUFF, 9, 0}, 
    {"saddlebrown", (void *) LXB_CSS_VALUE_SADDLEBROWN, 11, 86}, 
    {"inline", (void *) LXB_CSS_VALUE_INLINE, 6, 254}, 
    {"isolate", (void *) LXB_CSS_VALUE_ISOLATE, 7, 0}, 
    {"goldenrod", (void *) LXB_CSS_VALUE_GOLDENROD, 9, 0}, 
    {"cyan", (void *) LXB_CSS_VALUE_CYAN, 4, 256}, 
    {"linen", (void *) LXB_CSS_VALUE_LINEN, 5, 257}, 
    {"column", (void *) LXB_CSS_VALUE_COLUMN, 6, 258}, 
    {"crimson", (void *) LXB_CSS_VALUE_CRIMSON, 7, 259}, 
    {"springgreen", (void *) LXB_CSS_VALUE_SPRINGGREEN, 11, 0}, 
    {"lawngreen", (void *) LXB_CSS_VALUE_LAWNGREEN, 9, 260}, 
    {"lightgreen", (void *) LXB_CSS_VALUE_LIGHTGREEN, 10, 0}, 
    {"plum", (void *) LXB_CSS_VALUE_PLUM, 4, 262}, 
    {"initial", (void *) LXB_CSS_VALUE_INITIAL, 7, 264}, 
    {"lightseagreen", (void *) LXB_CSS_VALUE_LIGHTSEAGREEN, 13, 0}, 
    {"mediumblue", (void *) LXB_CSS_VALUE_MEDIUMBLUE, 10, 93}, 
    {"activetext", (void *) LXB_CSS_VALUE_ACTIVETEXT, 10, 94}, 
    {"buttonface", (void *) LXB_CSS_VALUE_BUTTONFACE, 10, 96}, 
    {"each-line", (void *) LXB_CSS_VALUE_EACH_LINE, 9, 0}, 
    {"greenyellow", (void *) LXB_CSS_VALUE_GREENYELLOW, 11, 0}, 
    {"hanging", (void *) LXB_CSS_VALUE_HANGING, 7, 0}, 
    {"hotpink", (void *) LXB_CSS_VALUE_HOTPINK, 7, 0}, 
    {"midnightblue", (void *) LXB_CSS_VALUE_MIDNIGHTBLUE, 12, 0}, 
    {"stretch", (void *) LXB_CSS_VALUE_STRETCH, 7, 0}, 
    {"lime", (void *) LXB_CSS_VALUE_LIME, 4, 266}, 
    {"unset", (void *) LXB_CSS_VALUE_UNSET, 5, 267}, 
    {"x-small", (void *) LXB_CSS_VALUE_X_SMALL, 7, 0}, 
    {"upright", (void *) LXB_CSS_VALUE_UPRIGHT, 7, 0}, 
    {"xx-small", (void *) LXB_CSS_VALUE_XX_SMALL, 8, 108}, 
    {"lightblue", (void *) LXB_CSS_VALUE_LIGHTBLUE, 9, 270}, 
    {"aqua", (void *) LXB_CSS_VALUE_AQUA, 4, 271}, 
    {"mediumslateblue", (void *) LXB_CSS_VALUE_MEDIUMSLATEBLUE, 15, 110}, 
    {"lightskyblue", (void *) LXB_CSS_VALUE_LIGHTSKYBLUE, 12, 0}, 
    {"mediumturquoise", (void *) LXB_CSS_VALUE_MEDIUMTURQUOISE, 15, 113}, 
    {"plaintext", (void *) LXB_CSS_VALUE_PLAINTEXT, 9, 272}, 
    {"justify-all", (void *) LXB_CSS_VALUE_JUSTIFY_ALL, 11, 0}, 
    {"accentcolortext", (void *) LXB_CSS_VALUE_ACCENTCOLORTEXT, 15, 0}, 
    {"tan", (void *) LXB_CSS_VALUE_TAN, 3, 273}, 
    {"thin", (void *) LXB_CSS_VALUE_THIN, 4, 274}, 
    {"blueviolet", (void *) LXB_CSS_VALUE_BLUEVIOLET, 10, 275}, 
    {"run-in", (void *) LXB_CSS_VALUE_RUN_IN, 6, 276}, 
    {"match-parent", (void *) LXB_CSS_VALUE_MATCH_PARENT, 12, 0}, 
    {"mediumaquamarine", (void *) LXB_CSS_VALUE_MEDIUMAQUAMARINE, 16, 0}, 
    {"snow", (void *) LXB_CSS_VALUE_SNOW, 4, 280}, 
    {"small", (void *) LXB_CSS_VALUE_SMALL, 5, 0}, 
    {"sienna", (void *) LXB_CSS_VALUE_SIENNA, 6, 283}, 
    {"table-column", (void *) LXB_CSS_VALUE_TABLE_COLUMN, 12, 0}, 
    {"seashell", (void *) LXB_CSS_VALUE_SEASHELL, 8, 286}, 
    {"lightslategray", (void *) LXB_CSS_VALUE_LIGHTSLATEGRAY, 14, 287}, 
    {"ruby", (void *) LXB_CSS_VALUE_RUBY, 4, 0}, 
    {"sideways-rl", (void *) LXB_CSS_VALUE_SIDEWAYS_RL, 11, 0}, 
    {"all", (void *) LXB_CSS_VALUE_ALL, 3, 288}, 
    {"extra-condensed", (void *) LXB_CSS_VALUE_EXTRA_CONDENSED, 15, 0}, 
    {"indigo", (void *) LXB_CSS_VALUE_INDIGO, 6, 0}, 
    {"block-end", (void *) LXB_CSS_VALUE_BLOCK_END, 9, 134}, 
    {"bolder", (void *) LXB_CSS_VALUE_BOLDER, 6, 0}, 
    {"forestgreen", (void *) LXB_CSS_VALUE_FORESTGREEN, 11, 0}, 
    {"burlywood", (void *) LXB_CSS_VALUE_BURLYWOOD, 9, 0}, 
    {"blanchedalmond", (void *) LXB_CSS_VALUE_BLANCHEDALMOND, 14, 0}, 
    {"mediumvioletred", (void *) LXB_CSS_VALUE_MEDIUMVIOLETRED, 15, 0}, 
    {"papayawhip", (void *) LXB_CSS_VALUE_PAPAYAWHIP, 10, 0}, 
    {"buttonborder", (void *) LXB_CSS_VALUE_BUTTONBORDER, 12, 0}, 
    {"nowrap", (void *) LXB_CSS_VALUE_NOWRAP, 6, 0}, 
    {"double", (void *) LXB_CSS_VALUE_DOUBLE, 6, 141}, 
    {"smaller", (void *) LXB_CSS_VALUE_SMALLER, 7, 142}, 
    {"cornsilk", (void *) LXB_CSS_VALUE_CORNSILK, 8, 0}, 
    {"dodgerblue", (void *) LXB_CSS_VALUE_DODGERBLUE, 10, 144}, 
    {"sideways-lr", (void *) LXB_CSS_VALUE_SIDEWAYS_LR, 11, 0}, 
    {"deepskyblue", (void *) LXB_CSS_VALUE_DEEPSKYBLUE, 11, 0}, 
    {"skyblue", (void *) LXB_CSS_VALUE_SKYBLUE, 7, 0}, 
    {"gainsboro", (void *) LXB_CSS_VALUE_GAINSBORO, 9, 0}, 
    {"khaki", (void *) LXB_CSS_VALUE_KHAKI, 5, 290}, 
    {"firebrick", (void *) LXB_CSS_VALUE_FIREBRICK, 9, 153}, 
    {"snap-inline", (void *) LXB_CSS_VALUE_SNAP_INLINE, 11, 0}, 
    {"dotted", (void *) LXB_CSS_VALUE_DOTTED, 6, 293}, 
    {"flow", (void *) LXB_CSS_VALUE_FLOW, 4, 295}, 
    {"darkslateblue", (void *) LXB_CSS_VALUE_DARKSLATEBLUE, 13, 154}, 
    {"darkturquoise", (void *) LXB_CSS_VALUE_DARKTURQUOISE, 13, 0}, 
    {"hwb", (void *) LXB_CSS_VALUE_HWB, 3, 297}, 
    {"highlight", (void *) LXB_CSS_VALUE_HIGHLIGHT, 9, 0}, 
    {"palegoldenrod", (void *) LXB_CSS_VALUE_PALEGOLDENROD, 13, 159}, 
    {"visible", (void *) LXB_CSS_VALUE_VISIBLE, 7, 298}, 
    {"palevioletred", (void *) LXB_CSS_VALUE_PALEVIOLETRED, 13, 0}, 
    {"green", (void *) LXB_CSS_VALUE_GREEN, 5, 299}, 
    {"hidden", (void *) LXB_CSS_VALUE_HIDDEN, 6, 300}, 
    {"emoji", (void *) LXB_CSS_VALUE_EMOJI, 5, 301}, 
    {"darkgray", (void *) LXB_CSS_VALUE_DARKGRAY, 8, 303}, 
    {"tomato", (void *) LXB_CSS_VALUE_TOMATO, 6, 304}, 
    {"xx-large", (void *) LXB_CSS_VALUE_XX_LARGE, 8, 305}, 
    {"fangsong", (void *) LXB_CSS_VALUE_FANGSONG, 8, 306}, 
    {"start", (void *) LXB_CSS_VALUE_START, 5, 0}, 
    {"right", (void *) LXB_CSS_VALUE_RIGHT, 5, 307}, 
    {"revert", (void *) LXB_CSS_VALUE_REVERT, 6, 0}, 
    {"wavy", (void *) LXB_CSS_VALUE_WAVY, 4, 0}, 
    {"rgba", (void *) LXB_CSS_VALUE_RGBA, 4, 310}, 
    {"maroon", (void *) LXB_CSS_VALUE_MAROON, 6, 313}, 
    {"olive", (void *) LXB_CSS_VALUE_OLIVE, 5, 314}, 
    {"orange", (void *) LXB_CSS_VALUE_ORANGE, 6, 315}, 
    {"oldlace", (void *) LXB_CSS_VALUE_OLDLACE, 7, 317}, 
    {"overline", (void *) LXB_CSS_VALUE_OVERLINE, 8, 320}, 
    {"pre", (void *) LXB_CSS_VALUE_PRE, 3, 322}, 
    {"lch", (void *) LXB_CSS_VALUE_LCH, 3, 323}, 
    {"ruby-base", (void *) LXB_CSS_VALUE_RUBY_BASE, 9, 0}, 
    {"purple", (void *) LXB_CSS_VALUE_PURPLE, 6, 325}, 
    {"rtl", (void *) LXB_CSS_VALUE_RTL, 3, 328}, 
    {"bottom", (void *) LXB_CSS_VALUE_BOTTOM, 6, 329}, 
    {"flex", (void *) LXB_CSS_VALUE_FLEX, 4, 331}, 
    {"powderblue", (void *) LXB_CSS_VALUE_POWDERBLUE, 10, 0}, 
    {"ltr", (void *) LXB_CSS_VALUE_LTR, 3, 334}, 
    {"ultra-condensed", (void *) LXB_CSS_VALUE_ULTRA_CONDENSED, 15, 0}, 
    {"line-through", (void *) LXB_CSS_VALUE_LINE_THROUGH, 12, 336}, 
    {"sub", (void *) LXB_CSS_VALUE_SUB, 3, 337}, 
    {"lighter", (void *) LXB_CSS_VALUE_LIGHTER, 7, 340}, 
    {"lavender", (void *) LXB_CSS_VALUE_LAVENDER, 8, 341}, 
    {"inline-grid", (void *) LXB_CSS_VALUE_INLINE_GRID, 11, 0}, 
    {"row", (void *) LXB_CSS_VALUE_ROW, 3, 343}, 
    {"row-reverse", (void *) LXB_CSS_VALUE_ROW_REVERSE, 11, 0}, 
    {"rebeccapurple", (void *) LXB_CSS_VALUE_REBECCAPURPLE, 13, 0}, 
    {"content", (void *) LXB_CSS_VALUE_CONTENT, 7, 344}, 
    {"red", (void *) LXB_CSS_VALUE_RED, 3, 0}, 
    {"selecteditem", (void *) LXB_CSS_VALUE_SELECTEDITEM, 12, 0}, 
    {"oklch", (void *) LXB_CSS_VALUE_OKLCH, 5, 346}, 
    {"table", (void *) LXB_CSS_VALUE_TABLE, 5, 348}, 
    {"mark", (void *) LXB_CSS_VALUE_MARK, 4, 0}, 
    {"black", (void *) LXB_CSS_VALUE_BLACK, 5, 349}, 
    {"inline-block", (void *) LXB_CSS_VALUE_INLINE_BLOCK, 12, 0}, 
    {"clip", (void *) LXB_CSS_VALUE_CLIP, 4, 352}, 
    {"anywhere", (void *) LXB_CSS_VALUE_ANYWHERE, 8, 205}, 
    {"absolute", (void *) LXB_CSS_VALUE_ABSOLUTE, 8, 0}, 
    {"wheat", (void *) LXB_CSS_VALUE_WHEAT, 5, 0}, 
    {"underline", (void *) LXB_CSS_VALUE_UNDERLINE, 9, 353}, 
    {"top", (void *) LXB_CSS_VALUE_TOP, 3, 0}, 
    {"alphabetic", (void *) LXB_CSS_VALUE_ALPHABETIC, 10, 0}, 
    {"ui-monospace", (void *) LXB_CSS_VALUE_UI_MONOSPACE, 12, 0}, 
    {"darkkhaki", (void *) LXB_CSS_VALUE_DARKKHAKI, 9, 0}, 
    {"graytext", (void *) LXB_CSS_VALUE_GRAYTEXT, 8, 214}, 
    {"serif", (void *) LXB_CSS_VALUE_SERIF, 5, 354}, 
    {"list-item", (void *) LXB_CSS_VALUE_LIST_ITEM, 9, 0}, 
    {"coral", (void *) LXB_CSS_VALUE_CORAL, 5, 355}, 
    {"ellipsis", (void *) LXB_CSS_VALUE_ELLIPSIS, 8, 0}, 
    {"central", (void *) LXB_CSS_VALUE_CENTRAL, 7, 357}, 
    {"darkcyan", (void *) LXB_CSS_VALUE_DARKCYAN, 8, 358}, 
    {"hsla", (void *) LXB_CSS_VALUE_HSLA, 4, 359}, 
    {"darksalmon", (void *) LXB_CSS_VALUE_DARKSALMON, 10, 360}, 
    {"flow-root", (void *) LXB_CSS_VALUE_FLOW_ROOT, 9, 0}, 
    {"darkseagreen", (void *) LXB_CSS_VALUE_DARKSEAGREEN, 12, 0}, 
    {"table-header-group", (void *) LXB_CSS_VALUE_TABLE_HEADER_GROUP, 18, 361}, 
    {"darkolivegreen", (void *) LXB_CSS_VALUE_DARKOLIVEGREEN, 14, 0}, 
    {"gray", (void *) LXB_CSS_VALUE_GRAY, 4, 363}, 
    {"_angle", (void *) LXB_CSS_VALUE__ANGLE, 6, 0}, 
    {"ghostwhite", (void *) LXB_CSS_VALUE_GHOSTWHITE, 10, 229}, 
    {"thick", (void *) LXB_CSS_VALUE_THICK, 5, 0}, 
    {"wrap-reverse", (void *) LXB_CSS_VALUE_WRAP_REVERSE, 12, 0}, 
    {"math", (void *) LXB_CSS_VALUE_MATH, 4, 365}, 
    {"_length", (void *) LXB_CSS_VALUE__LENGTH, 7, 366}, 
    {"fuchsia", (void *) LXB_CSS_VALUE_FUCHSIA, 7, 0}, 
    {"wrap", (void *) LXB_CSS_VALUE_WRAP, 4, 367}, 
    {"fixed", (void *) LXB_CSS_VALUE_FIXED, 5, 235}, 
    {"lightyellow", (void *) LXB_CSS_VALUE_LIGHTYELLOW, 11, 0}, 
    {"clear", (void *) LXB_CSS_VALUE_CLEAR, 5, 237}, 
    {"deeppink", (void *) LXB_CSS_VALUE_DEEPPINK, 8, 0}, 
    {"last", (void *) LXB_CSS_VALUE_LAST, 4, 368}, 
    {"full-size-kana", (void *) LXB_CSS_VALUE_FULL_SIZE_KANA, 14, 0}, 
    {"flex-end", (void *) LXB_CSS_VALUE_FLEX_END, 8, 0}, 
    {"palegreen", (void *) LXB_CSS_VALUE_PALEGREEN, 9, 370}, 
    {"linktext", (void *) LXB_CSS_VALUE_LINKTEXT, 8, 0}, 
    {"text-bottom", (void *) LXB_CSS_VALUE_TEXT_BOTTOM, 11, 245}, 
    {"inset", (void *) LXB_CSS_VALUE_INSET, 5, 0}, 
    {"inline-flex", (void *) LXB_CSS_VALUE_INLINE_FLEX, 11, 0}, 
    {"inherit", (void *) LXB_CSS_VALUE_INHERIT, 7, 0}, 
    {"navy", (void *) LXB_CSS_VALUE_NAVY, 4, 0}, 
    {"grid", (void *) LXB_CSS_VALUE_GRID, 4, 252}, 
    {"ruby-base-container", (void *) LXB_CSS_VALUE_RUBY_BASE_CONTAINER, 19, 371}, 
    {"orchid", (void *) LXB_CSS_VALUE_ORCHID, 6, 0}, 
    {"inline-start", (void *) LXB_CSS_VALUE_INLINE_START, 12, 0}, 
    {"border-box", (void *) LXB_CSS_VALUE_BORDER_BOX, 10, 0}, 
    {"peru", (void *) LXB_CSS_VALUE_PERU, 4, 372}, 
    {"currentcolor", (void *) LXB_CSS_VALUE_CURRENTCOLOR, 12, 255}, 
    {"lightgoldenrodyellow", (void *) LXB_CSS_VALUE_LIGHTGOLDENRODYELLOW, 20, 0}, 
    {"space-around", (void *) LXB_CSS_VALUE_SPACE_AROUND, 12, 0}, 
    {"semi-expanded", (void *) LXB_CSS_VALUE_SEMI_EXPANDED, 13, 0}, 
    {"semi-condensed", (void *) LXB_CSS_VALUE_SEMI_CONDENSED, 14, 0}, 
    {"inline-table", (void *) LXB_CSS_VALUE_INLINE_TABLE, 12, 0}, 
    {"lightcyan", (void *) LXB_CSS_VALUE_LIGHTCYAN, 9, 261}, 
    {"limegreen", (void *) LXB_CSS_VALUE_LIMEGREEN, 9, 0}, 
    {"lightsalmon", (void *) LXB_CSS_VALUE_LIGHTSALMON, 11, 263}, 
    {"isolate-override", (void *) LXB_CSS_VALUE_ISOLATE_OVERRIDE, 16, 0}, 
    {"keep-all", (void *) LXB_CSS_VALUE_KEEP_ALL, 8, 265}, 
    {"lemonchiffon", (void *) LXB_CSS_VALUE_LEMONCHIFFON, 12, 0}, 
    {"sideways", (void *) LXB_CSS_VALUE_SIDEWAYS, 8, 0}, 
    {"large", (void *) LXB_CSS_VALUE_LARGE, 5, 268}, 
    {"loose", (void *) LXB_CSS_VALUE_LOOSE, 5, 269}, 
    {"honeydew", (void *) LXB_CSS_VALUE_HONEYDEW, 8, 0}, 
    {"lowercase", (void *) LXB_CSS_VALUE_LOWERCASE, 9, 0}, 
    {"floralwhite", (void *) LXB_CSS_VALUE_FLORALWHITE, 11, 0}, 
    {"lightsteelblue", (void *) LXB_CSS_VALUE_LIGHTSTEELBLUE, 14, 0}, 
    {"marktext", (void *) LXB_CSS_VALUE_MARKTEXT, 8, 0}, 
    {"none", (void *) LXB_CSS_VALUE_NONE, 4, 0}, 
    {"buttontext", (void *) LXB_CSS_VALUE_BUTTONTEXT, 10, 0}, 
    {"region", (void *) LXB_CSS_VALUE_REGION, 6, 277}, 
    {"block-start", (void *) LXB_CSS_VALUE_BLOCK_START, 11, 278}, 
    {"min-content", (void *) LXB_CSS_VALUE_MIN_CONTENT, 11, 279}, 
    {"max-content", (void *) LXB_CSS_VALUE_MAX_CONTENT, 11, 0}, 
    {"lightgray", (void *) LXB_CSS_VALUE_LIGHTGRAY, 9, 281}, 
    {"lightgrey", (void *) LXB_CSS_VALUE_LIGHTGREY, 9, 282}, 
    {"rosybrown", (void *) LXB_CSS_VALUE_ROSYBROWN, 9, 0}, 
    {"scroll", (void *) LXB_CSS_VALUE_SCROLL, 6, 284}, 
    {"digits", (void *) LXB_CSS_VALUE_DIGITS, 6, 285}, 
    {"navajowhite", (void *) LXB_CSS_VALUE_NAVAJOWHITE, 11, 0}, 
    {"table-caption", (void *) LXB_CSS_VALUE_TABLE_CAPTION, 13, 0}, 
    {"lightslategrey", (void *) LXB_CSS_VALUE_LIGHTSLATEGREY, 14, 0}, 
    {"yellow", (void *) LXB_CSS_VALUE_YELLOW, 6, 289}, 
    {"break-spaces", (void *) LXB_CSS_VALUE_BREAK_SPACES, 12, 0}, 
    {"oklab", (void *) LXB_CSS_VALUE_OKLAB, 5, 291}, 
    {"slateblue", (void *) LXB_CSS_VALUE_SLATEBLUE, 9, 292}, 
    {"steelblue", (void *) LXB_CSS_VALUE_STEELBLUE, 9, 0}, 
    {"dashed", (void *) LXB_CSS_VALUE_DASHED, 6, 294}, 
    {"transparent", (void *) LXB_CSS_VALUE_TRANSPARENT, 11, 0}, 
    {"darkred", (void *) LXB_CSS_VALUE_DARKRED, 7, 296}, 
    {"olivedrab", (void *) LXB_CSS_VALUE_OLIVEDRAB, 9, 0}, 
    {"darkorchid", (void *) LXB_CSS_VALUE_DARKORCHID, 10, 0}, 
    {"darkgoldenrod", (void *) LXB_CSS_VALUE_DARKGOLDENROD, 13, 0}, 
    {"_number", (void *) LXB_CSS_VALUE__NUMBER, 7, 0}, 
    {"_integer", (void *) LXB_CSS_VALUE__INTEGER, 8, 0}, 
    {"dimgray", (void *) LXB_CSS_VALUE_DIMGRAY, 7, 302}, 
    {"dimgrey", (void *) LXB_CSS_VALUE_DIMGREY, 7, 0}, 
    {"darkgrey", (void *) LXB_CSS_VALUE_DARKGREY, 8, 0}, 
    {"x-large", (void *) LXB_CSS_VALUE_X_LARGE, 7, 0}, 
    {"horizontal-tb", (void *) LXB_CSS_VALUE_HORIZONTAL_TB, 13, 0}, 
    {"xxx-large", (void *) LXB_CSS_VALUE_XXX_LARGE, 9, 0}, 
    {"strict", (void *) LXB_CSS_VALUE_STRICT, 6, 308}, 
    {"darkslategray", (void *) LXB_CSS_VALUE_DARKSLATEGRAY, 13, 309}, 
    {"darkslategrey", (void *) LXB_CSS_VALUE_DARKSLATEGREY, 13, 0}, 
    {"brown", (void *) LXB_CSS_VALUE_BROWN, 5, 311}, 
    {"system-ui", (void *) LXB_CSS_VALUE_SYSTEM_UI, 9, 312}, 
    {"accentcolor", (void *) LXB_CSS_VALUE_ACCENTCOLOR, 11, 0}, 
    {"ruby-text", (void *) LXB_CSS_VALUE_RUBY_TEXT, 9, 0}, 
    {"cursive", (void *) LXB_CSS_VALUE_CURSIVE, 7, 0}, 
    {"moccasin", (void *) LXB_CSS_VALUE_MOCCASIN, 8, 316}, 
    {"collapse", (void *) LXB_CSS_VALUE_COLLAPSE, 8, 0}, 
    {"oblique", (void *) LXB_CSS_VALUE_OBLIQUE, 7, 318}, 
    {"cadetblue", (void *) LXB_CSS_VALUE_CADETBLUE, 9, 319}, 
    {"chocolate", (void *) LXB_CSS_VALUE_CHOCOLATE, 9, 0}, 
    {"chartreuse", (void *) LXB_CSS_VALUE_CHARTREUSE, 10, 321}, 
    {"capitalize", (void *) LXB_CSS_VALUE_CAPITALIZE, 10, 0}, 
    {"yellowgreen", (void *) LXB_CSS_VALUE_YELLOWGREEN, 11, 0}, 
    {"page", (void *) LXB_CSS_VALUE_PAGE, 4, 324}, 
    {"selecteditemtext", (void *) LXB_CSS_VALUE_SELECTEDITEMTEXT, 16, 0}, 
    {"cornflowerblue", (void *) LXB_CSS_VALUE_CORNFLOWERBLUE, 14, 326}, 
    {"mediumseagreen", (void *) LXB_CSS_VALUE_MEDIUMSEAGREEN, 14, 327}, 
    {"column-reverse", (void *) LXB_CSS_VALUE_COLUMN_REVERSE, 14, 0}, 
    {"ui-rounded", (void *) LXB_CSS_VALUE_UI_ROUNDED, 10, 0}, 
    {"medium", (void *) LXB_CSS_VALUE_MEDIUM, 6, 330}, 
    {"pre-line", (void *) LXB_CSS_VALUE_PRE_LINE, 8, 0}, 
    {"minimum", (void *) LXB_CSS_VALUE_MINIMUM, 7, 332}, 
    {"maximum", (void *) LXB_CSS_VALUE_MAXIMUM, 7, 333}, 
    {"mediumspringgreen", (void *) LXB_CSS_VALUE_MEDIUMSPRINGGREEN, 17, 0}, 
    {"mintcream", (void *) LXB_CSS_VALUE_MINTCREAM, 9, 335}, 
    {"ultra-expanded", (void *) LXB_CSS_VALUE_ULTRA_EXPANDED, 14, 0}, 
    {"paleturquoise", (void *) LXB_CSS_VALUE_PALETURQUOISE, 13, 0}, 
    {"larger", (void *) LXB_CSS_VALUE_LARGER, 6, 338}, 
    {"content-box", (void *) LXB_CSS_VALUE_CONTENT_BOX, 11, 339}, 
    {"lavenderblush", (void *) LXB_CSS_VALUE_LAVENDERBLUSH, 13, 0}, 
    {"indianred", (void *) LXB_CSS_VALUE_INDIANRED, 9, 0}, 
    {"inline-end", (void *) LXB_CSS_VALUE_INLINE_END, 10, 342}, 
    {"inter-word", (void *) LXB_CSS_VALUE_INTER_WORD, 10, 0}, 
    {"manual", (void *) LXB_CSS_VALUE_MANUAL, 6, 0}, 
    {"justify", (void *) LXB_CSS_VALUE_JUSTIFY, 7, 345}, 
    {"break-all", (void *) LXB_CSS_VALUE_BREAK_ALL, 9, 0}, 
    {"canvastext", (void *) LXB_CSS_VALUE_CANVASTEXT, 10, 347}, 
    {"mathematical", (void *) LXB_CSS_VALUE_MATHEMATICAL, 12, 0}, 
    {"full-width", (void *) LXB_CSS_VALUE_FULL_WIDTH, 10, 0}, 
    {"block", (void *) LXB_CSS_VALUE_BLOCK, 5, 350}, 
    {"blink", (void *) LXB_CSS_VALUE_BLINK, 5, 351}, 
    {"thistle", (void *) LXB_CSS_VALUE_THISTLE, 7, 0}, 
    {"turquoise", (void *) LXB_CSS_VALUE_TURQUOISE, 9, 0}, 
    {"uppercase", (void *) LXB_CSS_VALUE_UPPERCASE, 9, 0}, 
    {"text-top", (void *) LXB_CSS_VALUE_TEXT_TOP, 8, 0}, 
    {"ivory", (void *) LXB_CSS_VALUE_IVORY, 5, 356}, 
    {"inter-character", (void *) LXB_CSS_VALUE_INTER_CHARACTER, 15, 0}, 
    {"fantasy", (void *) LXB_CSS_VALUE_FANTASY, 7, 0}, 
    {"sans-serif", (void *) LXB_CSS_VALUE_SANS_SERIF, 10, 0}, 
    {"darkgreen", (void *) LXB_CSS_VALUE_DARKGREEN, 9, 0}, 
    {"table-row-group", (void *) LXB_CSS_VALUE_TABLE_ROW_GROUP, 15, 0}, 
    {"table-footer-group", (void *) LXB_CSS_VALUE_TABLE_FOOTER_GROUP, 18, 362}, 
    {"table-column-group", (void *) LXB_CSS_VALUE_TABLE_COLUMN_GROUP, 18, 0}, 
    {"grey", (void *) LXB_CSS_VALUE_GREY, 4, 364}, 
    {"vertical-rl", (void *) LXB_CSS_VALUE_VERTICAL_RL, 11, 0}, 
    {"both", (void *) LXB_CSS_VALUE_BOTH, 4, 0}, 
    {"_percentage", (void *) LXB_CSS_VALUE__PERCENTAGE, 11, 0}, 
    {"ui-serif", (void *) LXB_CSS_VALUE_UI_SERIF, 8, 0}, 
    {"left", (void *) LXB_CSS_VALUE_LEFT, 4, 369}, 
    {"ui-sans-serif", (void *) LXB_CSS_VALUE_UI_SANS_SERIF, 13, 0}, 
    {"condensed", (void *) LXB_CSS_VALUE_CONDENSED, 9, 0}, 
    {"ruby-text-container", (void *) LXB_CSS_VALUE_RUBY_TEXT_CONTAINER, 19, 0}, 
    {"orangered", (void *) LXB_CSS_VALUE_ORANGERED, 9, 0}
};


#endif /* LXB_CSS_VALUE_RES_H */
