/*
 * Copyright (C) 2019 Alexander Borisov
 *
 * Author: Alexander Borisov <borisov@lexbor.com>
 */

/*
 * Caution!
 * This file generated by the script "utils/lexbor/encoding/single-byte.py"!
 * Do not change this file!
 */


#ifndef LEXBOR_ENCODING_SINGLE_H
#define LEXBOR_ENCODING_SINGLE_H

#ifdef __cplusplus
extern "C" {
#endif

#include "lexbor/encoding/base.h"

#include "lexbor/core/shs.h"


#define LXB_ENCODING_SINGLE_HASH_IBM866_SIZE 344
#define LXB_ENCODING_SINGLE_HASH_ISO_8859_10_SIZE 343
#define LXB_ENCODING_SINGLE_HASH_ISO_8859_13_SIZE 345
#define LXB_ENCODING_SINGLE_HASH_ISO_8859_14_SIZE 407
#define LXB_ENCODING_SINGLE_HASH_ISO_8859_15_SIZE 344
#define LXB_ENCODING_SINGLE_HASH_ISO_8859_16_SIZE 413
#define LXB_ENCODING_SINGLE_HASH_ISO_8859_2_SIZE 368
#define LXB_ENCODING_SINGLE_HASH_ISO_8859_3_SIZE 343
#define LXB_ENCODING_SINGLE_HASH_ISO_8859_4_SIZE 343
#define LXB_ENCODING_SINGLE_HASH_ISO_8859_5_SIZE 343
#define LXB_ENCODING_SINGLE_HASH_ISO_8859_6_SIZE 343
#define LXB_ENCODING_SINGLE_HASH_ISO_8859_7_SIZE 344
#define LXB_ENCODING_SINGLE_HASH_ISO_8859_8_SIZE 347
#define LXB_ENCODING_SINGLE_HASH_KOI8_R_SIZE 486
#define LXB_ENCODING_SINGLE_HASH_KOI8_U_SIZE 380
#define LXB_ENCODING_SINGLE_HASH_MACINTOSH_SIZE 343
#define LXB_ENCODING_SINGLE_HASH_WINDOWS_1250_SIZE 432
#define LXB_ENCODING_SINGLE_HASH_WINDOWS_1251_SIZE 362
#define LXB_ENCODING_SINGLE_HASH_WINDOWS_1252_SIZE 373
#define LXB_ENCODING_SINGLE_HASH_WINDOWS_1253_SIZE 354
#define LXB_ENCODING_SINGLE_HASH_WINDOWS_1254_SIZE 354
#define LXB_ENCODING_SINGLE_HASH_WINDOWS_1255_SIZE 467
#define LXB_ENCODING_SINGLE_HASH_WINDOWS_1256_SIZE 343
#define LXB_ENCODING_SINGLE_HASH_WINDOWS_1257_SIZE 355
#define LXB_ENCODING_SINGLE_HASH_WINDOWS_1258_SIZE 405
#define LXB_ENCODING_SINGLE_HASH_WINDOWS_874_SIZE 359
#define LXB_ENCODING_SINGLE_HASH_X_MAC_CYRILLIC_SIZE 373

#ifndef LEXBOR_DISABLE_INTERNAL_EXTERN
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_ibm866[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_iso_8859_10[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_iso_8859_13[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_iso_8859_14[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_iso_8859_15[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_iso_8859_16[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_iso_8859_2[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_iso_8859_3[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_iso_8859_4[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_iso_8859_5[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_iso_8859_6[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_iso_8859_7[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_iso_8859_8[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_koi8_r[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_koi8_u[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_macintosh[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_windows_1250[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_windows_1251[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_windows_1252[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_windows_1253[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_windows_1254[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_windows_1255[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_windows_1256[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_windows_1257[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_windows_1258[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_windows_874[128];
LXB_EXTERN const lxb_encoding_single_index_t lxb_encoding_single_index_x_mac_cyrillic[128];

LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_ibm866[345];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_iso_8859_10[344];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_iso_8859_13[346];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_iso_8859_14[408];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_iso_8859_15[345];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_iso_8859_16[414];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_iso_8859_2[369];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_iso_8859_3[344];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_iso_8859_4[344];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_iso_8859_5[344];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_iso_8859_6[344];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_iso_8859_7[345];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_iso_8859_8[348];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_koi8_r[487];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_koi8_u[381];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_macintosh[351];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_windows_1250[433];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_windows_1251[363];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_windows_1252[374];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_windows_1253[355];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_windows_1254[355];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_windows_1255[468];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_windows_1256[357];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_windows_1257[356];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_windows_1258[406];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_windows_874[360];
LXB_EXTERN const lexbor_shs_hash_t lxb_encoding_single_hash_x_mac_cyrillic[374];
#endif /* !LEXBOR_DISABLE_INTERNAL_EXTERN */


#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* LEXBOR_ENCODING_SINGLE_H */
