<?php

/** @generate-class-entries */

namespace Pdo;

/**
 * @strict-properties
 * @not-serializable
 */
class Firebird extends \PDO
{
    /** @cvalue PDO_FB_ATTR_DATE_FORMAT */
    public const int ATTR_DATE_FORMAT = UNKNOWN;

    /** @cvalue PDO_FB_ATTR_TIME_FORMAT */
    public const int ATTR_TIME_FORMAT = UNKNOWN;

    /** @cvalue PDO_FB_ATTR_TIMESTAMP_FORMAT */
    public const int ATTR_TIMESTAMP_FORMAT = UNKNOWN;

    /** @cvalue PDO_FB_TRANSACTION_ISOLATION_LEVEL */
    public const int TRANSACTION_ISOLATION_LEVEL = UNKNOWN;

    /** @cvalue PDO_FB_READ_COMMITTED */
    public const int READ_COMMITTED = UNKNOWN;

    /** @cvalue PDO_FB_REPEATABLE_READ */
    public const int REPEATABLE_READ = UNKNOWN;

    /** @cvalue PDO_FB_SERIALIZABLE */
    public const int SERIALIZABLE = UNKNOWN;

    /** @cvalue PDO_FB_WRITABLE_TRANSACTION */
    public const int WRITABLE_TRANSACTION = UNKNOWN;

    public static function getApiVersion(): int {}
}
