/*
 * Copyright (C) 2026 Alexander Borisov
 *
 * Author: Alexander Borisov <borisov@lexbor.com>
 */

/*
 * Caution!
 * This file generated by the script "utils/lexbor/css/names.py"!
 * Do not change this file!
 */


#ifndef LXB_CSS_UNIT_CONST_H
#define LXB_CSS_UNIT_CONST_H


#include "lexbor/css/value/const.h"


typedef enum {
    LXB_CSS_UNIT__UNDEF      = 0x0000,
    LXB_CSS_UNIT__LAST_ENTRY = 0x0022
}
lxb_css_unit_t;

typedef enum {
    LXB_CSS_UNIT_ABSOLUTE__BEGIN      = 0x0001,
    LXB_CSS_UNIT_Q                    = 0x0001,
    LXB_CSS_UNIT_CM                   = 0x0002,
    LXB_CSS_UNIT_IN                   = 0x0003,
    LXB_CSS_UNIT_MM                   = 0x0004,
    LXB_CSS_UNIT_PC                   = 0x0005,
    LXB_CSS_UNIT_PT                   = 0x0006,
    LXB_CSS_UNIT_PX                   = 0x0007,
    LXB_CSS_UNIT_ABSOLUTE__LAST_ENTRY = 0x0008
}
lxb_css_unit_absolute_t;

typedef enum {
    LXB_CSS_UNIT_RELATIVE__BEGIN      = 0x0008,
    LXB_CSS_UNIT_CAP                  = 0x0008,
    LXB_CSS_UNIT_CH                   = 0x0009,
    LXB_CSS_UNIT_EM                   = 0x000a,
    LXB_CSS_UNIT_EX                   = 0x000b,
    LXB_CSS_UNIT_IC                   = 0x000c,
    LXB_CSS_UNIT_LH                   = 0x000d,
    LXB_CSS_UNIT_REM                  = 0x000e,
    LXB_CSS_UNIT_RLH                  = 0x000f,
    LXB_CSS_UNIT_VB                   = 0x0010,
    LXB_CSS_UNIT_VH                   = 0x0011,
    LXB_CSS_UNIT_VI                   = 0x0012,
    LXB_CSS_UNIT_VMAX                 = 0x0013,
    LXB_CSS_UNIT_VMIN                 = 0x0014,
    LXB_CSS_UNIT_VW                   = 0x0015,
    LXB_CSS_UNIT_RELATIVE__LAST_ENTRY = 0x0016
}
lxb_css_unit_relative_t;

typedef enum {
    LXB_CSS_UNIT_ANGLE__BEGIN      = 0x0016,
    LXB_CSS_UNIT_DEG               = 0x0016,
    LXB_CSS_UNIT_GRAD              = 0x0017,
    LXB_CSS_UNIT_RAD               = 0x0018,
    LXB_CSS_UNIT_TURN              = 0x0019,
    LXB_CSS_UNIT_ANGLE__LAST_ENTRY = 0x001a
}
lxb_css_unit_angle_t;

typedef enum {
    LXB_CSS_UNIT_FREQUENCY__BEGIN      = 0x001a,
    LXB_CSS_UNIT_HZ                    = 0x001a,
    LXB_CSS_UNIT_KHZ                   = 0x001b,
    LXB_CSS_UNIT_FREQUENCY__LAST_ENTRY = 0x001c
}
lxb_css_unit_frequency_t;

typedef enum {
    LXB_CSS_UNIT_RESOLUTION__BEGIN      = 0x001c,
    LXB_CSS_UNIT_DPCM                   = 0x001c,
    LXB_CSS_UNIT_DPI                    = 0x001d,
    LXB_CSS_UNIT_DPPX                   = 0x001e,
    LXB_CSS_UNIT_X                      = 0x001f,
    LXB_CSS_UNIT_RESOLUTION__LAST_ENTRY = 0x0020
}
lxb_css_unit_resolution_t;

typedef enum {
    LXB_CSS_UNIT_DURATION__BEGIN      = 0x0020,
    LXB_CSS_UNIT_MS                   = 0x0020,
    LXB_CSS_UNIT_S                    = 0x0021,
    LXB_CSS_UNIT_DURATION__LAST_ENTRY = 0x0022
}
lxb_css_unit_duration_t;



#endif /* LXB_CSS_UNIT_CONST_H */
