/*
 * Copyright (C) 2025 Alexander Borisov
 *
 * Author: Alexander Borisov <borisov@lexbor.com>
 */

/*
 * Caution!
 * This file generated by the script "utils/lexbor/css/names.py"!
 * Do not change this file!
 */


#ifndef LXB_CSS_PROPERTY_TYPES_H
#define LXB_CSS_PROPERTY_TYPES_H





#endif /* LXB_CSS_PROPERTY_TYPES_H */
