/*
 * Copyright (C) 2018-2026 Alexander Borisov
 *
 * Author: Alexander Borisov <borisov@lexbor.com>
 */

/*
 * Caution!
 * This file generated by the script "utils/lexbor/tag_ns/tags.py"!
 * Do not change this file!
 */


#ifndef LXB_HTML_INTERFACE_RES_H
#define LXB_HTML_INTERFACE_RES_H

#ifdef LXB_TAG_CONST_VERSION
#ifndef LXB_TAG_CONST_VERSION_5AB3094FB370521074947DC082575715
#error Mismatched tags version! See "lexbor/tag/const.h".
#endif /* LXB_TAG_CONST_VERSION_5AB3094FB370521074947DC082575715 */
#else
#error You need to include "lexbor/tag/const.h".
#endif /* LXB_TAG_CONST_VERSION */

#ifdef LXB_NS_CONST_VERSION
#ifndef LXB_NS_CONST_VERSION_253D4AFDA959234B48A478B956C3C777
#error Mismatched namespaces version! See "lexbor/ns/const.h".
#endif /* LXB_NS_CONST_VERSION_253D4AFDA959234B48A478B956C3C777 */
#else
#error You need to include "lexbor/ns/const.h".
#endif /* LXB_NS_CONST_VERSION */

#include "lexbor/dom/interfaces/cdata_section.h"
#include "lexbor/dom/interfaces/character_data.h"
#include "lexbor/dom/interfaces/comment.h"
#include "lexbor/dom/interfaces/document.h"
#include "lexbor/dom/interfaces/document_fragment.h"
#include "lexbor/dom/interfaces/document_type.h"
#include "lexbor/dom/interfaces/element.h"
#include "lexbor/dom/interfaces/event_target.h"
#include "lexbor/dom/interfaces/node.h"
#include "lexbor/dom/interfaces/processing_instruction.h"
#include "lexbor/dom/interfaces/shadow_root.h"
#include "lexbor/dom/interfaces/text.h"
#include "lexbor/html/interfaces/document.h"
#include "lexbor/html/interfaces/anchor_element.h"
#include "lexbor/html/interfaces/area_element.h"
#include "lexbor/html/interfaces/audio_element.h"
#include "lexbor/html/interfaces/br_element.h"
#include "lexbor/html/interfaces/base_element.h"
#include "lexbor/html/interfaces/body_element.h"
#include "lexbor/html/interfaces/button_element.h"
#include "lexbor/html/interfaces/canvas_element.h"
#include "lexbor/html/interfaces/d_list_element.h"
#include "lexbor/html/interfaces/data_element.h"
#include "lexbor/html/interfaces/data_list_element.h"
#include "lexbor/html/interfaces/details_element.h"
#include "lexbor/html/interfaces/dialog_element.h"
#include "lexbor/html/interfaces/directory_element.h"
#include "lexbor/html/interfaces/div_element.h"
#include "lexbor/html/interfaces/element.h"
#include "lexbor/html/interfaces/embed_element.h"
#include "lexbor/html/interfaces/field_set_element.h"
#include "lexbor/html/interfaces/font_element.h"
#include "lexbor/html/interfaces/form_element.h"
#include "lexbor/html/interfaces/frame_element.h"
#include "lexbor/html/interfaces/frame_set_element.h"
#include "lexbor/html/interfaces/hr_element.h"
#include "lexbor/html/interfaces/head_element.h"
#include "lexbor/html/interfaces/heading_element.h"
#include "lexbor/html/interfaces/html_element.h"
#include "lexbor/html/interfaces/iframe_element.h"
#include "lexbor/html/interfaces/image_element.h"
#include "lexbor/html/interfaces/input_element.h"
#include "lexbor/html/interfaces/li_element.h"
#include "lexbor/html/interfaces/label_element.h"
#include "lexbor/html/interfaces/legend_element.h"
#include "lexbor/html/interfaces/link_element.h"
#include "lexbor/html/interfaces/map_element.h"
#include "lexbor/html/interfaces/marquee_element.h"
#include "lexbor/html/interfaces/media_element.h"
#include "lexbor/html/interfaces/menu_element.h"
#include "lexbor/html/interfaces/meta_element.h"
#include "lexbor/html/interfaces/meter_element.h"
#include "lexbor/html/interfaces/mod_element.h"
#include "lexbor/html/interfaces/o_list_element.h"
#include "lexbor/html/interfaces/object_element.h"
#include "lexbor/html/interfaces/opt_group_element.h"
#include "lexbor/html/interfaces/option_element.h"
#include "lexbor/html/interfaces/output_element.h"
#include "lexbor/html/interfaces/paragraph_element.h"
#include "lexbor/html/interfaces/param_element.h"
#include "lexbor/html/interfaces/picture_element.h"
#include "lexbor/html/interfaces/pre_element.h"
#include "lexbor/html/interfaces/progress_element.h"
#include "lexbor/html/interfaces/quote_element.h"
#include "lexbor/html/interfaces/script_element.h"
#include "lexbor/html/interfaces/search_element.h"
#include "lexbor/html/interfaces/select_element.h"
#include "lexbor/html/interfaces/selectedcontent_element.h"
#include "lexbor/html/interfaces/slot_element.h"
#include "lexbor/html/interfaces/source_element.h"
#include "lexbor/html/interfaces/span_element.h"
#include "lexbor/html/interfaces/style_element.h"
#include "lexbor/html/interfaces/table_caption_element.h"
#include "lexbor/html/interfaces/table_cell_element.h"
#include "lexbor/html/interfaces/table_col_element.h"
#include "lexbor/html/interfaces/table_element.h"
#include "lexbor/html/interfaces/table_row_element.h"
#include "lexbor/html/interfaces/table_section_element.h"
#include "lexbor/html/interfaces/template_element.h"
#include "lexbor/html/interfaces/text_area_element.h"
#include "lexbor/html/interfaces/time_element.h"
#include "lexbor/html/interfaces/title_element.h"
#include "lexbor/html/interfaces/track_element.h"
#include "lexbor/html/interfaces/u_list_element.h"
#include "lexbor/html/interfaces/unknown_element.h"
#include "lexbor/html/interfaces/video_element.h"
#include "lexbor/html/interfaces/window.h"

lxb_inline void *
lxb_dom_element_interface_create_wrapper(void *interface)
{
    return lxb_dom_element_interface_create(interface);
}

lxb_inline void *
lxb_dom_element_interface_destroy_wrapper(void *interface)
{
    return lxb_dom_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_element_interface_create_wrapper(void *interface)
{
    return lxb_html_element_interface_create(interface);
}

lxb_inline void *
lxb_html_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_element_interface_destroy(interface);
}

lxb_inline void *
lxb_dom_text_interface_create_wrapper(void *interface)
{
    return lxb_dom_text_interface_create(interface);
}

lxb_inline void *
lxb_dom_text_interface_destroy_wrapper(void *interface)
{
    return lxb_dom_text_interface_destroy(interface);
}

lxb_inline void *
lxb_dom_document_interface_create_wrapper(void *interface)
{
    return lxb_dom_document_interface_create(interface);
}

lxb_inline void *
lxb_dom_document_interface_destroy_wrapper(void *interface)
{
    return lxb_dom_document_interface_destroy(interface);
}

lxb_inline void *
lxb_html_document_interface_create_wrapper(void *interface)
{
    return lxb_html_document_interface_create(interface);
}

lxb_inline void *
lxb_html_document_interface_destroy_wrapper(void *interface)
{
    return lxb_html_document_interface_destroy(interface);
}

lxb_inline void *
lxb_dom_comment_interface_create_wrapper(void *interface)
{
    return lxb_dom_comment_interface_create(interface);
}

lxb_inline void *
lxb_dom_comment_interface_destroy_wrapper(void *interface)
{
    return lxb_dom_comment_interface_destroy(interface);
}

lxb_inline void *
lxb_dom_document_type_interface_create_wrapper(void *interface)
{
    return lxb_dom_document_type_interface_create(interface);
}

lxb_inline void *
lxb_dom_document_type_interface_destroy_wrapper(void *interface)
{
    return lxb_dom_document_type_interface_destroy(interface);
}

lxb_inline void *
lxb_html_anchor_element_interface_create_wrapper(void *interface)
{
    return lxb_html_anchor_element_interface_create(interface);
}

lxb_inline void *
lxb_html_anchor_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_anchor_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_unknown_element_interface_create_wrapper(void *interface)
{
    return lxb_html_unknown_element_interface_create(interface);
}

lxb_inline void *
lxb_html_unknown_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_unknown_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_area_element_interface_create_wrapper(void *interface)
{
    return lxb_html_area_element_interface_create(interface);
}

lxb_inline void *
lxb_html_area_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_area_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_audio_element_interface_create_wrapper(void *interface)
{
    return lxb_html_audio_element_interface_create(interface);
}

lxb_inline void *
lxb_html_audio_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_audio_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_base_element_interface_create_wrapper(void *interface)
{
    return lxb_html_base_element_interface_create(interface);
}

lxb_inline void *
lxb_html_base_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_base_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_quote_element_interface_create_wrapper(void *interface)
{
    return lxb_html_quote_element_interface_create(interface);
}

lxb_inline void *
lxb_html_quote_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_quote_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_body_element_interface_create_wrapper(void *interface)
{
    return lxb_html_body_element_interface_create(interface);
}

lxb_inline void *
lxb_html_body_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_body_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_br_element_interface_create_wrapper(void *interface)
{
    return lxb_html_br_element_interface_create(interface);
}

lxb_inline void *
lxb_html_br_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_br_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_button_element_interface_create_wrapper(void *interface)
{
    return lxb_html_button_element_interface_create(interface);
}

lxb_inline void *
lxb_html_button_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_button_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_canvas_element_interface_create_wrapper(void *interface)
{
    return lxb_html_canvas_element_interface_create(interface);
}

lxb_inline void *
lxb_html_canvas_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_canvas_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_table_caption_element_interface_create_wrapper(void *interface)
{
    return lxb_html_table_caption_element_interface_create(interface);
}

lxb_inline void *
lxb_html_table_caption_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_table_caption_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_table_col_element_interface_create_wrapper(void *interface)
{
    return lxb_html_table_col_element_interface_create(interface);
}

lxb_inline void *
lxb_html_table_col_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_table_col_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_data_element_interface_create_wrapper(void *interface)
{
    return lxb_html_data_element_interface_create(interface);
}

lxb_inline void *
lxb_html_data_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_data_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_data_list_element_interface_create_wrapper(void *interface)
{
    return lxb_html_data_list_element_interface_create(interface);
}

lxb_inline void *
lxb_html_data_list_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_data_list_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_mod_element_interface_create_wrapper(void *interface)
{
    return lxb_html_mod_element_interface_create(interface);
}

lxb_inline void *
lxb_html_mod_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_mod_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_details_element_interface_create_wrapper(void *interface)
{
    return lxb_html_details_element_interface_create(interface);
}

lxb_inline void *
lxb_html_details_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_details_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_dialog_element_interface_create_wrapper(void *interface)
{
    return lxb_html_dialog_element_interface_create(interface);
}

lxb_inline void *
lxb_html_dialog_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_dialog_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_directory_element_interface_create_wrapper(void *interface)
{
    return lxb_html_directory_element_interface_create(interface);
}

lxb_inline void *
lxb_html_directory_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_directory_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_div_element_interface_create_wrapper(void *interface)
{
    return lxb_html_div_element_interface_create(interface);
}

lxb_inline void *
lxb_html_div_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_div_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_d_list_element_interface_create_wrapper(void *interface)
{
    return lxb_html_d_list_element_interface_create(interface);
}

lxb_inline void *
lxb_html_d_list_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_d_list_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_embed_element_interface_create_wrapper(void *interface)
{
    return lxb_html_embed_element_interface_create(interface);
}

lxb_inline void *
lxb_html_embed_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_embed_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_field_set_element_interface_create_wrapper(void *interface)
{
    return lxb_html_field_set_element_interface_create(interface);
}

lxb_inline void *
lxb_html_field_set_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_field_set_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_font_element_interface_create_wrapper(void *interface)
{
    return lxb_html_font_element_interface_create(interface);
}

lxb_inline void *
lxb_html_font_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_font_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_form_element_interface_create_wrapper(void *interface)
{
    return lxb_html_form_element_interface_create(interface);
}

lxb_inline void *
lxb_html_form_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_form_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_frame_element_interface_create_wrapper(void *interface)
{
    return lxb_html_frame_element_interface_create(interface);
}

lxb_inline void *
lxb_html_frame_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_frame_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_frame_set_element_interface_create_wrapper(void *interface)
{
    return lxb_html_frame_set_element_interface_create(interface);
}

lxb_inline void *
lxb_html_frame_set_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_frame_set_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_heading_element_interface_create_wrapper(void *interface)
{
    return lxb_html_heading_element_interface_create(interface);
}

lxb_inline void *
lxb_html_heading_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_heading_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_head_element_interface_create_wrapper(void *interface)
{
    return lxb_html_head_element_interface_create(interface);
}

lxb_inline void *
lxb_html_head_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_head_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_hr_element_interface_create_wrapper(void *interface)
{
    return lxb_html_hr_element_interface_create(interface);
}

lxb_inline void *
lxb_html_hr_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_hr_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_html_element_interface_create_wrapper(void *interface)
{
    return lxb_html_html_element_interface_create(interface);
}

lxb_inline void *
lxb_html_html_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_html_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_iframe_element_interface_create_wrapper(void *interface)
{
    return lxb_html_iframe_element_interface_create(interface);
}

lxb_inline void *
lxb_html_iframe_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_iframe_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_image_element_interface_create_wrapper(void *interface)
{
    return lxb_html_image_element_interface_create(interface);
}

lxb_inline void *
lxb_html_image_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_image_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_input_element_interface_create_wrapper(void *interface)
{
    return lxb_html_input_element_interface_create(interface);
}

lxb_inline void *
lxb_html_input_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_input_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_label_element_interface_create_wrapper(void *interface)
{
    return lxb_html_label_element_interface_create(interface);
}

lxb_inline void *
lxb_html_label_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_label_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_legend_element_interface_create_wrapper(void *interface)
{
    return lxb_html_legend_element_interface_create(interface);
}

lxb_inline void *
lxb_html_legend_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_legend_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_li_element_interface_create_wrapper(void *interface)
{
    return lxb_html_li_element_interface_create(interface);
}

lxb_inline void *
lxb_html_li_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_li_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_link_element_interface_create_wrapper(void *interface)
{
    return lxb_html_link_element_interface_create(interface);
}

lxb_inline void *
lxb_html_link_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_link_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_pre_element_interface_create_wrapper(void *interface)
{
    return lxb_html_pre_element_interface_create(interface);
}

lxb_inline void *
lxb_html_pre_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_pre_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_map_element_interface_create_wrapper(void *interface)
{
    return lxb_html_map_element_interface_create(interface);
}

lxb_inline void *
lxb_html_map_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_map_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_marquee_element_interface_create_wrapper(void *interface)
{
    return lxb_html_marquee_element_interface_create(interface);
}

lxb_inline void *
lxb_html_marquee_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_marquee_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_menu_element_interface_create_wrapper(void *interface)
{
    return lxb_html_menu_element_interface_create(interface);
}

lxb_inline void *
lxb_html_menu_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_menu_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_meta_element_interface_create_wrapper(void *interface)
{
    return lxb_html_meta_element_interface_create(interface);
}

lxb_inline void *
lxb_html_meta_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_meta_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_meter_element_interface_create_wrapper(void *interface)
{
    return lxb_html_meter_element_interface_create(interface);
}

lxb_inline void *
lxb_html_meter_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_meter_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_object_element_interface_create_wrapper(void *interface)
{
    return lxb_html_object_element_interface_create(interface);
}

lxb_inline void *
lxb_html_object_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_object_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_o_list_element_interface_create_wrapper(void *interface)
{
    return lxb_html_o_list_element_interface_create(interface);
}

lxb_inline void *
lxb_html_o_list_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_o_list_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_opt_group_element_interface_create_wrapper(void *interface)
{
    return lxb_html_opt_group_element_interface_create(interface);
}

lxb_inline void *
lxb_html_opt_group_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_opt_group_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_option_element_interface_create_wrapper(void *interface)
{
    return lxb_html_option_element_interface_create(interface);
}

lxb_inline void *
lxb_html_option_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_option_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_output_element_interface_create_wrapper(void *interface)
{
    return lxb_html_output_element_interface_create(interface);
}

lxb_inline void *
lxb_html_output_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_output_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_paragraph_element_interface_create_wrapper(void *interface)
{
    return lxb_html_paragraph_element_interface_create(interface);
}

lxb_inline void *
lxb_html_paragraph_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_paragraph_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_param_element_interface_create_wrapper(void *interface)
{
    return lxb_html_param_element_interface_create(interface);
}

lxb_inline void *
lxb_html_param_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_param_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_picture_element_interface_create_wrapper(void *interface)
{
    return lxb_html_picture_element_interface_create(interface);
}

lxb_inline void *
lxb_html_picture_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_picture_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_progress_element_interface_create_wrapper(void *interface)
{
    return lxb_html_progress_element_interface_create(interface);
}

lxb_inline void *
lxb_html_progress_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_progress_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_script_element_interface_create_wrapper(void *interface)
{
    return lxb_html_script_element_interface_create(interface);
}

lxb_inline void *
lxb_html_script_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_script_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_search_element_interface_create_wrapper(void *interface)
{
    return lxb_html_search_element_interface_create(interface);
}

lxb_inline void *
lxb_html_search_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_search_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_select_element_interface_create_wrapper(void *interface)
{
    return lxb_html_select_element_interface_create(interface);
}

lxb_inline void *
lxb_html_select_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_select_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_selectedcontent_element_interface_create_wrapper(void *interface)
{
    return lxb_html_selectedcontent_element_interface_create(interface);
}

lxb_inline void *
lxb_html_selectedcontent_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_selectedcontent_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_slot_element_interface_create_wrapper(void *interface)
{
    return lxb_html_slot_element_interface_create(interface);
}

lxb_inline void *
lxb_html_slot_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_slot_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_source_element_interface_create_wrapper(void *interface)
{
    return lxb_html_source_element_interface_create(interface);
}

lxb_inline void *
lxb_html_source_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_source_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_span_element_interface_create_wrapper(void *interface)
{
    return lxb_html_span_element_interface_create(interface);
}

lxb_inline void *
lxb_html_span_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_span_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_style_element_interface_create_wrapper(void *interface)
{
    return lxb_html_style_element_interface_create(interface);
}

lxb_inline void *
lxb_html_style_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_style_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_table_element_interface_create_wrapper(void *interface)
{
    return lxb_html_table_element_interface_create(interface);
}

lxb_inline void *
lxb_html_table_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_table_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_table_section_element_interface_create_wrapper(void *interface)
{
    return lxb_html_table_section_element_interface_create(interface);
}

lxb_inline void *
lxb_html_table_section_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_table_section_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_table_cell_element_interface_create_wrapper(void *interface)
{
    return lxb_html_table_cell_element_interface_create(interface);
}

lxb_inline void *
lxb_html_table_cell_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_table_cell_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_template_element_interface_create_wrapper(void *interface)
{
    return lxb_html_template_element_interface_create(interface);
}

lxb_inline void *
lxb_html_template_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_template_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_text_area_element_interface_create_wrapper(void *interface)
{
    return lxb_html_text_area_element_interface_create(interface);
}

lxb_inline void *
lxb_html_text_area_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_text_area_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_time_element_interface_create_wrapper(void *interface)
{
    return lxb_html_time_element_interface_create(interface);
}

lxb_inline void *
lxb_html_time_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_time_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_title_element_interface_create_wrapper(void *interface)
{
    return lxb_html_title_element_interface_create(interface);
}

lxb_inline void *
lxb_html_title_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_title_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_table_row_element_interface_create_wrapper(void *interface)
{
    return lxb_html_table_row_element_interface_create(interface);
}

lxb_inline void *
lxb_html_table_row_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_table_row_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_track_element_interface_create_wrapper(void *interface)
{
    return lxb_html_track_element_interface_create(interface);
}

lxb_inline void *
lxb_html_track_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_track_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_u_list_element_interface_create_wrapper(void *interface)
{
    return lxb_html_u_list_element_interface_create(interface);
}

lxb_inline void *
lxb_html_u_list_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_u_list_element_interface_destroy(interface);
}

lxb_inline void *
lxb_html_video_element_interface_create_wrapper(void *interface)
{
    return lxb_html_video_element_interface_create(interface);
}

lxb_inline void *
lxb_html_video_element_interface_destroy_wrapper(void *interface)
{
    return lxb_html_video_element_interface_destroy(interface);
}


static  lxb_dom_interface_constructor_f lxb_html_interface_res_constructors[LXB_TAG__LAST_ENTRY][LXB_NS__LAST_ENTRY] = 
{
    /* LXB_TAG__UNDEF */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG__END_OF_FILE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG__TEXT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_text_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_text_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_text_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_text_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_text_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_text_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_text_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_text_interface_create_wrapper
    },
    /* LXB_TAG__DOCUMENT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_document_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_document_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_document_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_document_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_document_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_document_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_document_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_document_interface_create_wrapper
    },
    /* LXB_TAG__EM_COMMENT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_comment_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_comment_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_comment_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_comment_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_comment_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_comment_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_comment_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_comment_interface_create_wrapper
    },
    /* LXB_TAG__EM_DOCTYPE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_document_type_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_document_type_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_document_type_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_document_type_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_document_type_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_document_type_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_document_type_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_document_type_interface_create_wrapper
    },
    /* LXB_TAG_A */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_anchor_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_ABBR */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_ACRONYM */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_ADDRESS */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_ALTGLYPH */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_ALTGLYPHDEF */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_ALTGLYPHITEM */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_ANIMATECOLOR */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_ANIMATEMOTION */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_ANIMATETRANSFORM */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_ANNOTATION_XML */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_APPLET */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_AREA */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_area_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_ARTICLE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_ASIDE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_AUDIO */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_audio_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_B */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_BASE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_base_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_BASEFONT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_BDI */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_BDO */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_BGSOUND */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_BIG */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_BLINK */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_BLOCKQUOTE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_quote_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_BODY */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_body_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_BR */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_br_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_BUTTON */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_button_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_CANVAS */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_canvas_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_CAPTION */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_table_caption_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_CENTER */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_CITE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_CLIPPATH */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_CODE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_COL */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_table_col_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_COLGROUP */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_table_col_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_DATA */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_data_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_DATALIST */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_data_list_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_DD */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_DEL */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_mod_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_DESC */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_DETAILS */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_details_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_DFN */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_DIALOG */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_dialog_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_DIR */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_directory_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_DIV */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_div_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_DL */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_d_list_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_DT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_EM */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_EMBED */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_embed_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FEBLEND */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FECOLORMATRIX */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FECOMPONENTTRANSFER */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FECOMPOSITE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FECONVOLVEMATRIX */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FEDIFFUSELIGHTING */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FEDISPLACEMENTMAP */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FEDISTANTLIGHT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FEDROPSHADOW */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FEFLOOD */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FEFUNCA */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FEFUNCB */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FEFUNCG */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FEFUNCR */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FEGAUSSIANBLUR */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FEIMAGE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FEMERGE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FEMERGENODE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FEMORPHOLOGY */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FEOFFSET */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FEPOINTLIGHT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FESPECULARLIGHTING */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FESPOTLIGHT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FETILE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FETURBULENCE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FIELDSET */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_field_set_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FIGCAPTION */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FIGURE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FONT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_font_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FOOTER */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FOREIGNOBJECT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FORM */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_form_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FRAME */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_frame_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_FRAMESET */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_frame_set_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_GLYPHREF */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_H1 */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_heading_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_H2 */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_heading_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_H3 */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_heading_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_H4 */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_heading_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_H5 */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_heading_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_H6 */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_heading_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_HEAD */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_head_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_HEADER */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_HGROUP */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_HR */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_hr_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_HTML */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_I */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_IFRAME */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_iframe_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_IMAGE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_image_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_IMG */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_image_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_INPUT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_input_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_INS */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_mod_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_ISINDEX */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_KBD */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_KEYGEN */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_LABEL */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_label_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_LEGEND */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_legend_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_LI */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_li_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_LINEARGRADIENT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_LINK */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_link_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_LISTING */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_pre_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_MAIN */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_MALIGNMARK */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_MAP */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_map_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_MARK */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_MARQUEE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_marquee_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_MATH */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_MENU */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_menu_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_META */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_meta_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_METER */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_meter_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_MFENCED */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_MGLYPH */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_MI */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_MN */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_MO */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_MS */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_MTEXT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_MULTICOL */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_NAV */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_NEXTID */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_NOBR */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_NOEMBED */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_NOFRAMES */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_NOSCRIPT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_OBJECT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_object_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_OL */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_o_list_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_OPTGROUP */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_opt_group_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_OPTION */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_option_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_OUTPUT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_output_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_P */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_paragraph_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_PARAM */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_param_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_PATH */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_PICTURE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_picture_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_PLAINTEXT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_PRE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_pre_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_PROGRESS */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_progress_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_Q */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_quote_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_RADIALGRADIENT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_RB */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_RP */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_RT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_RTC */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_RUBY */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_S */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_SAMP */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_SCRIPT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_script_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_SEARCH */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_search_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_SECTION */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_SELECT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_select_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_SELECTEDCONTENT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_selectedcontent_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_SLOT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_slot_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_SMALL */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_SOURCE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_source_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_SPACER */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_SPAN */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_span_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_STRIKE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_STRONG */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_STYLE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_style_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_SUB */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_SUMMARY */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_SUP */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_SVG */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_TABLE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_table_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_TBODY */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_table_section_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_TD */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_table_cell_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_TEMPLATE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_template_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_TEXTAREA */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_text_area_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_TEXTPATH */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_unknown_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_TFOOT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_table_section_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_TH */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_table_cell_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_THEAD */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_table_section_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_TIME */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_time_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_TITLE */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_title_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_TR */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_table_row_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_TRACK */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_track_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_TT */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_U */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_UL */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_u_list_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_VAR */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_VIDEO */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_video_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_WBR */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    },
    /* LXB_TAG_XMP */
    {
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_html_pre_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper,
        (lxb_dom_interface_constructor_f) lxb_dom_element_interface_create_wrapper
    }
};

static  lxb_dom_interface_destructor_f lxb_html_interface_res_destructor[LXB_TAG__LAST_ENTRY][LXB_NS__LAST_ENTRY] = 
{
    /* LXB_TAG__UNDEF */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG__END_OF_FILE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG__TEXT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_text_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_text_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_text_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_text_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_text_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_text_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_text_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_text_interface_destroy_wrapper
    },
    /* LXB_TAG__DOCUMENT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_document_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_document_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_document_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_document_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_document_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_document_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_document_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_document_interface_destroy_wrapper
    },
    /* LXB_TAG__EM_COMMENT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_comment_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_comment_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_comment_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_comment_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_comment_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_comment_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_comment_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_comment_interface_destroy_wrapper
    },
    /* LXB_TAG__EM_DOCTYPE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_document_type_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_document_type_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_document_type_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_document_type_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_document_type_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_document_type_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_document_type_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_document_type_interface_destroy_wrapper
    },
    /* LXB_TAG_A */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_anchor_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_ABBR */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_ACRONYM */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_ADDRESS */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_ALTGLYPH */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_ALTGLYPHDEF */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_ALTGLYPHITEM */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_ANIMATECOLOR */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_ANIMATEMOTION */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_ANIMATETRANSFORM */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_ANNOTATION_XML */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_APPLET */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_AREA */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_area_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_ARTICLE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_ASIDE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_AUDIO */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_audio_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_B */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_BASE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_base_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_BASEFONT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_BDI */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_BDO */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_BGSOUND */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_BIG */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_BLINK */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_BLOCKQUOTE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_quote_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_BODY */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_body_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_BR */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_br_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_BUTTON */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_button_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_CANVAS */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_canvas_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_CAPTION */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_table_caption_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_CENTER */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_CITE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_CLIPPATH */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_CODE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_COL */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_table_col_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_COLGROUP */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_table_col_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_DATA */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_data_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_DATALIST */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_data_list_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_DD */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_DEL */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_mod_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_DESC */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_DETAILS */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_details_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_DFN */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_DIALOG */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_dialog_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_DIR */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_directory_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_DIV */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_div_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_DL */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_d_list_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_DT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_EM */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_EMBED */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_embed_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FEBLEND */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FECOLORMATRIX */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FECOMPONENTTRANSFER */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FECOMPOSITE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FECONVOLVEMATRIX */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FEDIFFUSELIGHTING */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FEDISPLACEMENTMAP */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FEDISTANTLIGHT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FEDROPSHADOW */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FEFLOOD */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FEFUNCA */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FEFUNCB */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FEFUNCG */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FEFUNCR */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FEGAUSSIANBLUR */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FEIMAGE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FEMERGE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FEMERGENODE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FEMORPHOLOGY */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FEOFFSET */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FEPOINTLIGHT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FESPECULARLIGHTING */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FESPOTLIGHT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FETILE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FETURBULENCE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FIELDSET */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_field_set_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FIGCAPTION */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FIGURE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FONT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_font_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FOOTER */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FOREIGNOBJECT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FORM */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_form_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FRAME */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_frame_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_FRAMESET */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_frame_set_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_GLYPHREF */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_H1 */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_heading_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_H2 */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_heading_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_H3 */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_heading_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_H4 */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_heading_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_H5 */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_heading_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_H6 */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_heading_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_HEAD */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_head_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_HEADER */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_HGROUP */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_HR */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_hr_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_HTML */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_I */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_IFRAME */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_iframe_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_IMAGE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_image_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_IMG */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_image_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_INPUT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_input_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_INS */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_mod_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_ISINDEX */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_KBD */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_KEYGEN */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_LABEL */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_label_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_LEGEND */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_legend_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_LI */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_li_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_LINEARGRADIENT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_LINK */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_link_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_LISTING */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_pre_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_MAIN */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_MALIGNMARK */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_MAP */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_map_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_MARK */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_MARQUEE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_marquee_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_MATH */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_MENU */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_menu_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_META */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_meta_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_METER */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_meter_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_MFENCED */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_MGLYPH */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_MI */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_MN */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_MO */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_MS */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_MTEXT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_MULTICOL */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_NAV */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_NEXTID */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_NOBR */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_NOEMBED */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_NOFRAMES */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_NOSCRIPT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_OBJECT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_object_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_OL */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_o_list_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_OPTGROUP */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_opt_group_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_OPTION */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_option_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_OUTPUT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_output_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_P */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_paragraph_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_PARAM */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_param_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_PATH */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_PICTURE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_picture_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_PLAINTEXT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_PRE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_pre_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_PROGRESS */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_progress_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_Q */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_quote_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_RADIALGRADIENT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_RB */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_RP */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_RT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_RTC */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_RUBY */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_S */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_SAMP */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_SCRIPT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_script_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_SEARCH */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_search_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_SECTION */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_SELECT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_select_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_SELECTEDCONTENT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_selectedcontent_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_SLOT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_slot_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_SMALL */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_SOURCE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_source_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_SPACER */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_SPAN */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_span_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_STRIKE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_STRONG */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_STYLE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_style_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_SUB */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_SUMMARY */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_SUP */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_SVG */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_TABLE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_table_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_TBODY */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_table_section_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_TD */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_table_cell_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_TEMPLATE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_template_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_TEXTAREA */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_text_area_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_TEXTPATH */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_unknown_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_TFOOT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_table_section_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_TH */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_table_cell_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_THEAD */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_table_section_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_TIME */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_time_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_TITLE */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_title_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_TR */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_table_row_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_TRACK */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_track_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_TT */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_U */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_UL */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_u_list_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_VAR */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_VIDEO */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_video_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_WBR */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    },
    /* LXB_TAG_XMP */
    {
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_html_pre_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper,
        (lxb_dom_interface_destructor_f) lxb_dom_element_interface_destroy_wrapper
    }
};

#endif /* LXB_HTML_INTERFACE_RES_H */
